# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/model.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from rastervision.protos.tf_object_detection import faster_rcnn_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_faster__rcnn__pb2
from rastervision.protos.tf_object_detection import ssd_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_ssd__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/model.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n3rastervision/protos/tf_object_detection/model.proto\x12\'rastervision.protos.tf_object_detection\x1a\x39rastervision/protos/tf_object_detection/faster_rcnn.proto\x1a\x31rastervision/protos/tf_object_detection/ssd.proto\"\xa2\x01\n\x0e\x44\x65tectionModel\x12J\n\x0b\x66\x61ster_rcnn\x18\x01 \x01(\x0b\x32\x33.rastervision.protos.tf_object_detection.FasterRcnnH\x00\x12;\n\x03ssd\x18\x02 \x01(\x0b\x32,.rastervision.protos.tf_object_detection.SsdH\x00\x42\x07\n\x05model')
  ,
  dependencies=[rastervision_dot_protos_dot_tf__object__detection_dot_faster__rcnn__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_ssd__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_DETECTIONMODEL = _descriptor.Descriptor(
  name='DetectionModel',
  full_name='rastervision.protos.tf_object_detection.DetectionModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='faster_rcnn', full_name='rastervision.protos.tf_object_detection.DetectionModel.faster_rcnn', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ssd', full_name='rastervision.protos.tf_object_detection.DetectionModel.ssd', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='model', full_name='rastervision.protos.tf_object_detection.DetectionModel.model',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=207,
  serialized_end=369,
)

_DETECTIONMODEL.fields_by_name['faster_rcnn'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_faster__rcnn__pb2._FASTERRCNN
_DETECTIONMODEL.fields_by_name['ssd'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_ssd__pb2._SSD
_DETECTIONMODEL.oneofs_by_name['model'].fields.append(
  _DETECTIONMODEL.fields_by_name['faster_rcnn'])
_DETECTIONMODEL.fields_by_name['faster_rcnn'].containing_oneof = _DETECTIONMODEL.oneofs_by_name['model']
_DETECTIONMODEL.oneofs_by_name['model'].fields.append(
  _DETECTIONMODEL.fields_by_name['ssd'])
_DETECTIONMODEL.fields_by_name['ssd'].containing_oneof = _DETECTIONMODEL.oneofs_by_name['model']
DESCRIPTOR.message_types_by_name['DetectionModel'] = _DETECTIONMODEL

DetectionModel = _reflection.GeneratedProtocolMessageType('DetectionModel', (_message.Message,), dict(
  DESCRIPTOR = _DETECTIONMODEL,
  __module__ = 'rastervision.protos.tf_object_detection.model_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.DetectionModel)
  ))
_sym_db.RegisterMessage(DetectionModel)


# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod


class Backend(ABC):
    """Functionality for a specific implementation of an MLTask.

    This should be subclassed to provide a bridge to third party ML libraries.
    There is a many-to-one relationship from backends to tasks.
    """

    @abstractmethod
    def process_scene_data(self, scene, data, tmp_dir):
        """Process each scene's training data

        Args:
            scene: Scene
            data: TrainingData

        Returns:
            backend-specific data-structures consumed by backend's
            process_sceneset_results
        """
        pass

    @abstractmethod
    def process_sceneset_results(self, training_results, validation_results,
                                 tmp_dir):
        """After all scenes have been processed, process the resultset

        Args:
            training_results: dependent on the ml_backend's process_scene_data
            validation_results: dependent on the ml_backend's
                process_scene_data
        """
        pass

    @abstractmethod
    def train(self, tmp_dir):
        """Train a model.
        """
        pass

    @abstractmethod
    def load_model(self, tmp_dir):
        """Load the model in preparation for one or more prediction calls."""
        pass

    @abstractmethod
    def predict(self, chips, windows, tmp_dir):
        """Return predictions for a chip using model.

        Args:
            chips: [[height, width, channels], ...] numpy array of chips
            windows: List of boxes that are the windows aligned with the chips.

        Return:
            Labels object containing predictions
        """
        pass



# flake8: noqa

from rastervision.data.crs_transformer.crs_transformer import *
from rastervision.data.crs_transformer.identity_crs_transformer import *
from rastervision.data.crs_transformer.rasterio_crs_transformer import *



# flake8: noqa

from rastervision.data.vector_source.api import GEOJSON_SOURCE

# Registry Keys
RASTER_SOURCE = 'RASTER_SOURCE'

# For backward compatibility
GEOTIFF_SOURCE = 'GEOTIFF_SOURCE'
IMAGE_SOURCE = 'IMAGE_SOURCE'

RASTERIO_SOURCE = 'RASTERIO_SOURCE'
RASTERIZED_SOURCE = 'RASTERIZED_SOURCE'

raster_source_deprecated_map = {
    GEOJSON_SOURCE: RASTERIZED_SOURCE,
    GEOTIFF_SOURCE: RASTERIO_SOURCE,
    IMAGE_SOURCE: RASTERIO_SOURCE
}

from .raster_source_config import RasterSourceConfig



import json

from rastervision.data.vector_source.vector_source import VectorSource
from rastervision.utils.files import file_to_str


class GeoJSONVectorSource(VectorSource):
    def __init__(self,
                 uri,
                 crs_transformer,
                 line_bufs=None,
                 point_bufs=None,
                 class_inf_opts=None):
        """Constructor.

        Args:
            uri: (str) uri of GeoJSON file
            class_inf_opts: ClassInferenceOptions
        """
        self.uri = uri
        super().__init__(crs_transformer, line_bufs, point_bufs,
                         class_inf_opts)

    def _get_geojson(self):
        geojson = json.loads(file_to_str(self.uri))
        return self.class_inference.transform_geojson(geojson)



"""Main entry point"""
import sys

import rastervision as rv

# Code taken from unittest
if sys.argv[0].endswith('__main__.py'):
    import os.path
    # We change sys.argv[0] to make help message more useful
    # use executable without path, unquoted
    # (it's just a hint anyway)
    # (if you have spaces in your executable you get what you deserve!)
    executable = os.path.basename(sys.executable)
    sys.argv[0] = executable + ' -m rastervision'
    del os

rv.main()



# flake8: noqa

from rastervision.backend.backend import *
from rastervision.backend.backend_config import *

try:
    import tensorflow
    tf_available = True
except ImportError as e:
    tf_available = False

try:
    import torch
    pytorch_available = True
except ImportError as e:
    pytorch_available = False

if tf_available:
    from rastervision.backend.tf_object_detection import TFObjectDetection
    from rastervision.backend.tf_object_detection_config import (
        TFObjectDetectionConfig, TFObjectDetectionConfigBuilder)
    from rastervision.backend.keras_classification import KerasClassification
    from rastervision.backend.keras_classification_config import (
        KerasClassificationConfig, KerasClassificationConfigBuilder)
    from rastervision.backend.tf_deeplab import TFDeeplab
    from rastervision.backend.tf_deeplab_config import (TFDeeplabConfig,
                                                        TFDeeplabConfigBuilder)

if pytorch_available:
    from rastervision.backend.pytorch_chip_classification import (
        PyTorchChipClassification)
    from rastervision.backend.pytorch_chip_classification_config import (
        PyTorchChipClassificationConfig,
        PyTorchChipClassificationConfigBuilder)
    from rastervision.backend.pytorch_semantic_segmentation import (
        PyTorchSemanticSegmentation)
    from rastervision.backend.pytorch_semantic_segmentation_config import (
        PyTorchSemanticSegmentationConfig,
        PyTorchSemanticSegmentationConfigBuilder)
    from rastervision.backend.pytorch_object_detection_config import (
        PyTorchObjectDetectionConfig, PyTorchObjectDetectionConfigBuilder)



# flake8: noqa

from rastervision.task.api import (OBJECT_DETECTION, CHIP_CLASSIFICATION,
                                   SEMANTIC_SEGMENTATION)

# Registry Keys
LABEL_SOURCE = 'LABEL_SOURCE'

# Deprecated keys provided for backward compatibility.
OBJECT_DETECTION_GEOJSON = 'OBJECT_DETECTION_GEOJSON'
CHIP_CLASSIFICATION_GEOJSON = 'CHIP_CLASSIFICATION_GEOJSON'
SEMANTIC_SEGMENTATION_RASTER = 'SEMANTIC_SEGMENTATION_RASTER'

label_source_deprecated_map = {
    OBJECT_DETECTION_GEOJSON: OBJECT_DETECTION,
    CHIP_CLASSIFICATION_GEOJSON: CHIP_CLASSIFICATION,
    SEMANTIC_SEGMENTATION_RASTER: SEMANTIC_SEGMENTATION
}

from rastervision.data.label_source.label_source_config import LabelSourceConfig



from abc import (ABC, abstractmethod)
import copy

import json

from rastervision.evaluation import ClassEvaluationItem
from rastervision.utils.files import str_to_file


class ClassificationEvaluation(ABC):
    """Base class for evaluating predictions for tasks that have classes.

    Evaluations can be keyed, for instance, if evaluations happen per class.
    """

    def __init__(self):
        self.clear()
        self._is_empty = True

    def clear(self):
        """Clear the Evaluation."""
        self.class_to_eval_item = {}
        self.scene_to_eval = {}
        self.avg_item = None
        self._is_empty = True

    def is_empty(self):
        return self._is_empty

    def set_class_to_eval_item(self, class_to_eval_item):
        self.class_to_eval_item = class_to_eval_item

    def get_by_id(self, key):
        """Gets the evaluation for a particular EvaluationItem key"""
        return self.class_to_eval_item[key]

    def has_id(self, key):
        """Answers whether or not the EvaluationItem key is represented"""
        return key in self.class_to_eval_item

    def to_json(self):
        json_rep = []
        for eval_item in self.class_to_eval_item.values():
            json_rep.append(eval_item.to_json())
        if self.avg_item:
            json_rep.append(self.avg_item.to_json())

        if self.scene_to_eval:
            json_rep = {'overall': json_rep}
            scene_to_eval_json = {}
            for scene_id, eval in self.scene_to_eval.items():
                scene_to_eval_json[scene_id] = eval.to_json()
            json_rep['per_scene'] = scene_to_eval_json

        return json_rep

    def save(self, output_uri):
        """Save this Evaluation to a file.

        Args:
            output_uri: string URI for the file to write.
        """
        json_str = json.dumps(self.to_json(), indent=4)
        str_to_file(json_str, output_uri)

    def merge(self, evaluation, scene_id=None):
        """Merge Evaluation for another Scene into this one.

        This is useful for computing the average metrics of a set of scenes.
        The results of the averaging are stored in this Evaluation.

        Args:
            evaluation: Evaluation to merge into this one
        """
        if len(self.class_to_eval_item) == 0:
            self.class_to_eval_item = evaluation.class_to_eval_item
        else:
            for key, other_eval_item in \
                    evaluation.class_to_eval_item.items():
                if self.has_id(key):
                    self.get_by_id(key).merge(other_eval_item)
                else:
                    self.class_to_eval_item[key] = other_eval_item

        self._is_empty = False
        self.compute_avg()

        if scene_id is not None:
            self.scene_to_eval[scene_id] = copy.deepcopy(evaluation)

    def compute_avg(self):
        """Compute average metrics over all keys."""
        self.avg_item = ClassEvaluationItem(class_name='average')
        for eval_item in self.class_to_eval_item.values():
            self.avg_item.merge(eval_item)

    @abstractmethod
    def compute(self, ground_truth_labels, prediction_labels):
        """Compute metrics for a single scene.

        Args:
            ground_truth_labels: Ground Truth labels to evaluate against.
            prediction_labels: The predicted labels to evaluate.
        """
        pass



from copy import deepcopy

import rastervision as rv
from rastervision.command import (TrainCommand, CommandConfig,
                                  CommandConfigBuilder)
from rastervision.protos.command_pb2 \
    import CommandConfig as CommandConfigMsg
from rastervision.rv_config import RVConfig
from rastervision.command.utils import (check_task_type, check_backend_type)


class TrainCommandConfig(CommandConfig):
    def __init__(self, root_uri, task, backend):
        super().__init__(rv.TRAIN, root_uri)
        self.task = task
        self.backend = backend

    def utilizes_gpu(self):
        # TODO: Make this backend dependent.
        return True

    def create_command(self, tmp_dir=None):
        if not tmp_dir:
            _tmp_dir = RVConfig.get_tmp_dir()
            tmp_dir = _tmp_dir.name
        else:
            _tmp_dir = tmp_dir

        retval = TrainCommand(self)
        retval.set_tmp_dir(_tmp_dir)
        return retval

    def to_proto(self):
        msg = super().to_proto()

        task = self.task.to_proto()
        backend = self.backend.to_proto()

        msg.MergeFrom(
            CommandConfigMsg(
                train_config=CommandConfigMsg.TrainConfig(
                    task=task, backend=backend)))

        return msg

    def report_io(self):
        io_def = rv.core.CommandIODefinition()
        self.task.report_io(self.command_type, io_def)
        self.backend.report_io(self.command_type, io_def)
        return io_def


class TrainCommandConfigBuilder(CommandConfigBuilder):
    def __init__(self, command_type, prev=None):
        super().__init__(command_type, prev)
        if prev is None:
            self.task = None
            self.backend = None
        else:
            self.task = prev.task
            self.backend = prev.backend

    def validate(self):
        super().validate()
        if self.task is None:
            raise rv.ConfigError('Task not set for TrainCommandConfig. Use '
                                 'with_task or with_experiment')
        check_task_type(self.task)

        if self.backend is None:
            raise rv.ConfigError('Backend not set for TrainCommandConfig. Use '
                                 'with_task or with_experiment')
        check_backend_type(self.backend)

    def build(self):
        self.validate()
        return TrainCommandConfig(self.root_uri, self.task, self.backend)

    def from_proto(self, msg):
        b = super().from_proto(msg)

        conf = msg.train_config

        task = rv.TaskConfig.from_proto(conf.task)
        backend = rv.BackendConfig.from_proto(conf.backend)

        b = b.with_task(task)
        b = b.with_backend(backend)

        return b

    def get_root_uri(self, experiment_config):
        return experiment_config.train_uri

    def with_experiment(self, experiment_config):
        b = super().with_experiment(experiment_config)
        b = b.with_task(experiment_config.task)
        b = b.with_backend(experiment_config.backend)
        return b

    def with_task(self, task):
        """Sets a specific task type.

        Args:
            task:  A TaskConfig object.

        """
        b = deepcopy(self)
        b.task = task
        return b

    def with_backend(self, backend):
        b = deepcopy(self)
        b.backend = backend
        return b



import os
from copy import deepcopy

import rastervision as rv
from rastervision.core.raster_stats import RasterStats
from rastervision.data.raster_transformer import (
    RasterTransformerConfig, RasterTransformerConfigBuilder, StatsTransformer,
    NoopTransformer)
from rastervision.protos.raster_transformer_pb2 \
    import RasterTransformerConfig as RasterTransformerConfigMsg


class StatsTransformerConfig(RasterTransformerConfig):
    def __init__(self, stats_uri=None):
        super().__init__(rv.STATS_TRANSFORMER)
        self.stats_uri = stats_uri

    def to_proto(self):
        msg = RasterTransformerConfigMsg(
            transformer_type=self.transformer_type, stats_uri=self.stats_uri)
        return msg

    def save_bundle_files(self, bundle_dir):
        if not self.stats_uri:
            raise rv.ConfigError('stat_uri is not set.')
        local_path, base_name = self.bundle_file(self.stats_uri, bundle_dir)
        new_config = self.to_builder() \
                         .with_stats_uri(base_name) \
                         .build()
        return (new_config, [local_path])

    def load_bundle_files(self, bundle_dir):
        if not self.stats_uri:
            raise rv.ConfigError('stat_uri is not set.')
        local_stats_uri = os.path.join(bundle_dir, self.stats_uri)
        return self.to_builder() \
                   .with_stats_uri(local_stats_uri) \
                   .build()

    def create_transformer(self):
        if not self.stats_uri:
            return NoopTransformer()

        return StatsTransformer(RasterStats.load(self.stats_uri))

    def update_for_command(self, command_type, experiment_config,
                           context=None):
        if command_type != rv.ANALYZE:
            if not self.stats_uri:
                # Find the stats URI from a StatsAnalyzer
                for analyzer in experiment_config.analyzers:
                    if analyzer.analyzer_type == rv.STATS_ANALYZER:
                        self.stats_uri = analyzer.stats_uri

    def report_io(self, command_type, io_def):
        if command_type != rv.ANALYZE:
            if not self.stats_uri:
                io_def.add_missing(
                    "StatsTransformerConfig is missing 'stats_uri' property "
                    'in command {}. '
                    'This must be set on the configuration, or a '
                    'StatsAnalyzerConfig must be added to '
                    'this experiment.'.format(command_type))
            else:
                io_def.add_input(self.stats_uri)


class StatsTransformerConfigBuilder(RasterTransformerConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {'stats_uri': prev.stats_uri}
        super().__init__(StatsTransformerConfig, config)

    def from_proto(self, msg):
        return self.with_stats_uri(msg.stats_uri)

    def with_stats_uri(self, stats_uri):
        """Set the stats_uri.

            Args:
                stats_uri: URI to the stats json to use
        """
        b = deepcopy(self)
        b.config['stats_uri'] = stats_uri
        return b



import os
from os.path import join
import shutil
import gzip
from threading import Timer
import time
import logging
import json
import zipfile

from google.protobuf import json_format

from rastervision.filesystem.filesystem import FileSystem
from rastervision.filesystem.filesystem import ProtobufParseException
from rastervision.filesystem.local_filesystem import make_dir

log = logging.getLogger(__name__)


def get_local_path(uri, download_dir, fs=None):
    """Convert a URI into a corresponding local path.

    If a uri is local, return it. If it's remote, we generate a path for it
    within download_dir. For an S3 path of form s3://<bucket>/<key>, the path
    is <download_dir>/s3/<bucket>/<key>.

    Args:
        uri: (string) URI of file
        download_dir: (string) path to directory
        fs: Optional FileSystem to use

    Returns:
        (string) a local path
    """
    if uri is None:
        return None

    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')
    path = fs.local_path(uri, download_dir)

    return path


def sync_to_dir(src_dir_uri, dest_dir_uri, delete=False, fs=None):
    """Synchronize a local to a local or remote directory.

    Transfers files from source to destination directories so that the
    destination has all the source files. If delete is True, also delete
    files in the destination to match those in the source directory.

    Args:
        src_dir_uri: (string) URI of local source directory
        dest_dir_uri: (string) URI of destination directory
        delete: (bool)
        fs: Optional FileSystem to use for destination
    """
    if not fs:
        fs = FileSystem.get_file_system(dest_dir_uri, 'w')
    fs.sync_to_dir(src_dir_uri, dest_dir_uri, delete=delete)


def sync_from_dir(src_dir_uri, dest_dir_uri, delete=False, fs=None):
    """Synchronize a local or remote directory to a local directory.

    Transfers files from source to destination directories so that the
    destination has all the source files. If delete is True, also delete
    files in the destination to match those in the source directory.

    Args:
        src_dir_uri: (string) URI of source directory
        dest_dir_uri: (string) URI of local destination directory
        delete: (bool)
        fs: Optional FileSystem to use
    """
    if not fs:
        fs = FileSystem.get_file_system(src_dir_uri, 'r')
    fs.sync_from_dir(src_dir_uri, dest_dir_uri, delete=delete)


def start_sync(src_dir_uri, dest_dir_uri, sync_interval=600,
               fs=None):  # pragma: no cover
    """Start syncing a directory on a schedule.

    Calls sync_to_dir on a schedule.

    Args:
        src_dir_uri: (string) Path of the local source directory
        dest_dir_uri: (string) URI of destination directory
        sync_interval: (int) period in seconds for syncing
        fs:  Optional FileSystem to use
    """

    def _sync_dir():
        while True:
            time.sleep(sync_interval)
            log.info('Syncing {} to {}...'.format(src_dir_uri, dest_dir_uri))
            sync_to_dir(src_dir_uri, dest_dir_uri, delete=False, fs=fs)

    class SyncThread:
        def __init__(self):
            thread = Timer(0.68, _sync_dir)
            thread.daemon = True
            thread.start()
            self.thread = thread

        def __enter__(self):
            return self.thread

        def __exit__(self, type, value, traceback):
            self.thread.cancel()

    return SyncThread()


def download_if_needed(uri, download_dir, fs=None):
    """Download a file into a directory if it's remote.

    If uri is local, there is no need to download the file.

    Args:
        uri: (string) URI of file
        download_dir: (string) local directory to download file into
        fs: Optional FileSystem to use.

    Returns:
        (string) path to local file

    Raises:
        NotReadableError if URI cannot be read from
    """
    if uri is None:
        return None

    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')

    path = get_local_path(uri, download_dir, fs=fs)
    make_dir(path, use_dirname=True)

    if path != uri:
        log.info('Downloading {} to {}'.format(uri, path))

    fs.copy_from(uri, path)

    return path


def download_or_copy(uri, target_dir, fs=None):
    """Downloads or copies a file to a directory

    Args:
       uri: (string) URI of file
       target_dir: (string) local directory to copy file to
       fs: Optional FileSystem to use
    """
    local_path = download_if_needed(uri, target_dir, fs=fs)
    shutil.copy(local_path, target_dir)
    return local_path


def file_exists(uri, fs=None, include_dir=True):
    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')
    return fs.file_exists(uri, include_dir)


def list_paths(uri, ext='', fs=None):
    if uri is None:
        return None

    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')

    return fs.list_paths(uri, ext=ext)


def upload_or_copy(src_path, dst_uri, fs=None):
    """Upload a file if the destination is remote.

    If dst_uri is local, the file is copied.

    Args:
        src_path: (string) path to source file
        dst_uri: (string) URI of destination for file
        fs: Optional FileSystem to use
    Raises:
        NotWritableError if URI cannot be written to
    """
    if dst_uri is None:
        return

    if not (os.path.isfile(src_path) or os.path.isdir(src_path)):
        raise Exception('{} does not exist.'.format(src_path))

    if not src_path == dst_uri:
        log.info('Uploading {} to {}'.format(src_path, dst_uri))

    if not fs:
        fs = FileSystem.get_file_system(dst_uri, 'w')
    fs.copy_to(src_path, dst_uri)


def file_to_str(uri, fs=None):
    """Download contents of text file into a string.

    Args:
        uri: (string) URI of file
        fs: Optional FileSystem to use

    Returns:
        (string) with contents of text file

    Raises:
        NotReadableError if URI cannot be read from
    """
    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')
    return fs.read_str(uri)


def str_to_file(content_str, uri, fs=None):
    """Writes string to text file.

    Args:
        content_str: string to write
        uri: (string) URI of file to write
        fs: Optional FileSystem to use

    Raise:
        NotWritableError if file_uri cannot be written
    """
    if not fs:
        fs = FileSystem.get_file_system(uri, 'r')
    return fs.write_str(uri, content_str)


def load_json_config(uri, message, fs=None):
    """Load a JSON-formatted protobuf config file.

    Args:
        uri: (string) URI of config file
        message: (google.protobuf.message.Message) empty protobuf message of
            to load the config into. The type needs to match the content of
            uri.
        fs: Optional FileSystem to use.

    Returns:
        the same message passed as input with fields filled in from uri

    Raises:
        ProtobufParseException if uri cannot be parsed
    """
    try:
        return json_format.Parse(file_to_str(uri, fs=fs), message)
    except json_format.ParseError as e:
        error_msg = ('Problem parsing protobuf file {}. '.format(uri) +
                     'You might need to run scripts/compile')
        raise ProtobufParseException(error_msg) from e


def save_json_config(message, uri, fs=None):
    """Save a protobuf object to a JSON file.

    Args:
        message: (google.protobuf.message.Message) protobuf message
        uri: (string) URI of JSON file to write message to
        fs: Optional FileSystem to use

    Raises:
        NotWritableError if uri cannot be written
    """
    json_str = json_format.MessageToJson(message)
    str_to_file(json_str, uri, fs=fs)


def get_cached_file(cache_dir, uri):
    """Download a file and unzip it, using a cache to avoid unnecessary operations.

    This downloads a file if it isn't already in the cache, and unzips the file using
    gunzip if it hasn't already been unzipped (and the uri has a .gz suffix).

    Args:
        cache_dir: (str) dir to use for cache directory
        uri: (str) URI of a file that can be opened by a supported RV file system

    Returns:
        (str) path of the (downloaded and unzipped) cached file
    """
    # Only download if it isn't in the cache.
    path = get_local_path(uri, cache_dir)
    if not os.path.isfile(path):
        path = download_if_needed(uri, cache_dir)

    # Unzip if .gz file
    if path.endswith('.gz'):
        # If local URI, then make ungz_path in temp cache, so it isn't unzipped
        # alongside the original file.
        if os.path.isfile(uri):
            ungz_path = os.path.join(cache_dir, path)[:-3]
        else:
            ungz_path = path[:-3]

        # Check to see if it is already unzipped before unzipping.
        if not os.path.isfile(ungz_path):
            with gzip.open(path, 'rb') as f_in:
                with open(ungz_path, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
        path = ungz_path

    return path


def file_to_json(uri):
    """Return JSON dict based on file at uri."""
    return json.loads(file_to_str(uri))


def json_to_file(js, uri):
    """Upload file to uri based on JSON dict."""
    str_to_file(json.dumps(js), uri)


def zipdir(dir, zip_path):
    """Save a zip file with contents of directory.

    Contents of directory will be at root of zip file.

    Args:
        dir: (str) directory to zip
        zip_path: (str) path to zip file to create
    """
    make_dir(zip_path, use_dirname=True)
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as ziph:
        for dirpath, dirnames, filenames in os.walk(dir):
            for fn in filenames:
                ziph.write(join(dirpath, fn), join(dirpath[len(dir):], fn))


def unzip(zip_path, target_dir):
    """Unzip contents of zip file at zip_path into target_dir"""
    make_dir(target_dir)
    zip_ref = zipfile.ZipFile(zip_path, 'r')
    zip_ref.extractall(target_dir)
    zip_ref.close()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/keras_classification/model.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/keras_classification/model.proto',
  package='keras_classification.protos',
  syntax='proto2',
  serialized_pb=_b('\n4rastervision/protos/keras_classification/model.proto\x12\x1bkeras_classification.protos\"\xb5\x01\n\x05Model\x12\x12\n\ninput_size\x18\x01 \x02(\x05\x12\x12\n\nnb_classes\x18\x02 \x02(\x05\x12\x35\n\x04type\x18\x03 \x02(\x0e\x32\'.keras_classification.protos.Model.Type\x12\x12\n\nmodel_path\x18\x04 \x02(\t\x12#\n\x14load_weights_by_name\x18\x05 \x01(\x08:\x05\x66\x61lse\"\x14\n\x04Type\x12\x0c\n\x08RESNET50\x10\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_MODEL_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='keras_classification.protos.Model.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RESNET50', index=0, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=247,
  serialized_end=267,
)
_sym_db.RegisterEnumDescriptor(_MODEL_TYPE)


_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='keras_classification.protos.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input_size', full_name='keras_classification.protos.Model.input_size', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='nb_classes', full_name='keras_classification.protos.Model.nb_classes', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='keras_classification.protos.Model.type', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='model_path', full_name='keras_classification.protos.Model.model_path', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='load_weights_by_name', full_name='keras_classification.protos.Model.load_weights_by_name', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _MODEL_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=86,
  serialized_end=267,
)

_MODEL.fields_by_name['type'].enum_type = _MODEL_TYPE
_MODEL_TYPE.containing_type = _MODEL
DESCRIPTOR.message_types_by_name['Model'] = _MODEL

Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), dict(
  DESCRIPTOR = _MODEL,
  __module__ = 'rastervision.protos.keras_classification.model_pb2'
  # @@protoc_insertion_point(class_scope:keras_classification.protos.Model)
  ))
_sym_db.RegisterMessage(Model)


# @@protoc_insertion_point(module_scope)



import rastervision as rv
from rastervision.evaluation import SemanticSegmentationEvaluator
from rastervision.evaluation \
    import (ClassificationEvaluatorConfig, ClassificationEvaluatorConfigBuilder)


class SemanticSegmentationEvaluatorConfig(ClassificationEvaluatorConfig):
    def __init__(self, class_map, output_uri=None, vector_output_uri=None):
        super().__init__(rv.SEMANTIC_SEGMENTATION_EVALUATOR, class_map,
                         output_uri, vector_output_uri)

    def create_evaluator(self):
        return SemanticSegmentationEvaluator(self.class_map, self.output_uri,
                                             self.vector_output_uri)


class SemanticSegmentationEvaluatorConfigBuilder(
        ClassificationEvaluatorConfigBuilder):
    def __init__(self, prev=None):
        super().__init__(SemanticSegmentationEvaluatorConfig, prev)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/label_source.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from rastervision.protos import raster_source_pb2 as rastervision_dot_protos_dot_raster__source__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from rastervision.protos import class_item_pb2 as rastervision_dot_protos_dot_class__item__pb2
from rastervision.protos import vector_source_pb2 as rastervision_dot_protos_dot_vector__source__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/label_source.proto',
  package='rv.protos',
  syntax='proto2',
  serialized_pb=_b('\n&rastervision/protos/label_source.proto\x12\trv.protos\x1a\'rastervision/protos/raster_source.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a$rastervision/protos/class_item.proto\x1a\'rastervision/protos/vector_source.proto\"\xb3\x0c\n\x11LabelSourceConfig\x12\x13\n\x0bsource_type\x18\x01 \x02(\t\x12\x64\n\x1fobject_detection_geojson_source\x18\x02 \x01(\x0b\x32\x39.rv.protos.LabelSourceConfig.ObjectDetectionGeoJSONSourceH\x00\x12j\n\"chip_classification_geojson_source\x18\x03 \x01(\x0b\x32<.rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSourceH\x00\x12l\n#semantic_segmentation_raster_source\x18\x04 \x01(\x0b\x32=.rv.protos.LabelSourceConfig.SemanticSegmentationRasterSourceH\x00\x12\x30\n\rcustom_config\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x12`\n\x1dobject_detection_label_source\x18\x06 \x01(\x0b\x32\x37.rv.protos.LabelSourceConfig.ObjectDetectionLabelSourceH\x00\x12\x66\n chip_classification_label_source\x18\x07 \x01(\x0b\x32:.rv.protos.LabelSourceConfig.ChipClassificationLabelSourceH\x00\x12j\n\"semantic_segmentation_label_source\x18\x08 \x01(\x0b\x32<.rv.protos.LabelSourceConfig.SemanticSegmentationLabelSourceH\x00\x1aR\n\x1aObjectDetectionLabelSource\x12\x34\n\rvector_source\x18\x01 \x02(\x0b\x32\x1d.rv.protos.VectorSourceConfig\x1a\xf4\x01\n\x1d\x43hipClassificationLabelSource\x12\x34\n\rvector_source\x18\x01 \x02(\x0b\x32\x1d.rv.protos.VectorSourceConfig\x12\x12\n\nioa_thresh\x18\x02 \x01(\x02\x12\"\n\x1ause_intersection_over_cell\x18\x03 \x01(\x08\x12\x19\n\x11pick_min_class_id\x18\x04 \x01(\x08\x12\x1b\n\x13\x62\x61\x63kground_class_id\x18\x05 \x01(\x05\x12\x11\n\tcell_size\x18\x06 \x01(\x05\x12\x1a\n\x0binfer_cells\x18\x07 \x01(\x08:\x05\x66\x61lse\x1a\x7f\n\x1fSemanticSegmentationLabelSource\x12-\n\x06source\x18\x01 \x02(\x0b\x32\x1d.rv.protos.RasterSourceConfig\x12-\n\x0frgb_class_items\x18\x02 \x03(\x0b\x32\x14.rv.protos.ClassItem\x1a+\n\x1cObjectDetectionGeoJSONSource\x12\x0b\n\x03uri\x18\x01 \x02(\t\x1a\xcd\x01\n\x1f\x43hipClassificationGeoJSONSource\x12\x0b\n\x03uri\x18\x01 \x02(\t\x12\x12\n\nioa_thresh\x18\x02 \x01(\x02\x12\"\n\x1ause_intersection_over_cell\x18\x03 \x01(\x08\x12\x19\n\x11pick_min_class_id\x18\x04 \x01(\x08\x12\x1b\n\x13\x62\x61\x63kground_class_id\x18\x05 \x01(\x05\x12\x11\n\tcell_size\x18\x06 \x01(\x05\x12\x1a\n\x0binfer_cells\x18\x07 \x01(\x08:\x05\x66\x61lse\x1a\x80\x01\n SemanticSegmentationRasterSource\x12-\n\x06source\x18\x01 \x02(\x0b\x32\x1d.rv.protos.RasterSourceConfig\x12-\n\x0frgb_class_items\x18\x02 \x03(\x0b\x32\x14.rv.protos.ClassItemB\x15\n\x13label_source_config')
  ,
  dependencies=[rastervision_dot_protos_dot_raster__source__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,rastervision_dot_protos_dot_class__item__pb2.DESCRIPTOR,rastervision_dot_protos_dot_vector__source__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE = _descriptor.Descriptor(
  name='ObjectDetectionLabelSource',
  full_name='rv.protos.LabelSourceConfig.ObjectDetectionLabelSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vector_source', full_name='rv.protos.LabelSourceConfig.ObjectDetectionLabelSource.vector_source', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=926,
  serialized_end=1008,
)

_LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE = _descriptor.Descriptor(
  name='ChipClassificationLabelSource',
  full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vector_source', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.vector_source', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ioa_thresh', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.ioa_thresh', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_intersection_over_cell', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.use_intersection_over_cell', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pick_min_class_id', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.pick_min_class_id', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='background_class_id', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.background_class_id', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cell_size', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.cell_size', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='infer_cells', full_name='rv.protos.LabelSourceConfig.ChipClassificationLabelSource.infer_cells', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1011,
  serialized_end=1255,
)

_LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE = _descriptor.Descriptor(
  name='SemanticSegmentationLabelSource',
  full_name='rv.protos.LabelSourceConfig.SemanticSegmentationLabelSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='rv.protos.LabelSourceConfig.SemanticSegmentationLabelSource.source', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rgb_class_items', full_name='rv.protos.LabelSourceConfig.SemanticSegmentationLabelSource.rgb_class_items', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1257,
  serialized_end=1384,
)

_LABELSOURCECONFIG_OBJECTDETECTIONGEOJSONSOURCE = _descriptor.Descriptor(
  name='ObjectDetectionGeoJSONSource',
  full_name='rv.protos.LabelSourceConfig.ObjectDetectionGeoJSONSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='rv.protos.LabelSourceConfig.ObjectDetectionGeoJSONSource.uri', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1386,
  serialized_end=1429,
)

_LABELSOURCECONFIG_CHIPCLASSIFICATIONGEOJSONSOURCE = _descriptor.Descriptor(
  name='ChipClassificationGeoJSONSource',
  full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.uri', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ioa_thresh', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.ioa_thresh', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_intersection_over_cell', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.use_intersection_over_cell', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pick_min_class_id', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.pick_min_class_id', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='background_class_id', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.background_class_id', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cell_size', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.cell_size', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='infer_cells', full_name='rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource.infer_cells', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1432,
  serialized_end=1637,
)

_LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE = _descriptor.Descriptor(
  name='SemanticSegmentationRasterSource',
  full_name='rv.protos.LabelSourceConfig.SemanticSegmentationRasterSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='rv.protos.LabelSourceConfig.SemanticSegmentationRasterSource.source', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rgb_class_items', full_name='rv.protos.LabelSourceConfig.SemanticSegmentationRasterSource.rgb_class_items', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1640,
  serialized_end=1768,
)

_LABELSOURCECONFIG = _descriptor.Descriptor(
  name='LabelSourceConfig',
  full_name='rv.protos.LabelSourceConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source_type', full_name='rv.protos.LabelSourceConfig.source_type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='object_detection_geojson_source', full_name='rv.protos.LabelSourceConfig.object_detection_geojson_source', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='chip_classification_geojson_source', full_name='rv.protos.LabelSourceConfig.chip_classification_geojson_source', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='semantic_segmentation_raster_source', full_name='rv.protos.LabelSourceConfig.semantic_segmentation_raster_source', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='custom_config', full_name='rv.protos.LabelSourceConfig.custom_config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='object_detection_label_source', full_name='rv.protos.LabelSourceConfig.object_detection_label_source', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='chip_classification_label_source', full_name='rv.protos.LabelSourceConfig.chip_classification_label_source', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='semantic_segmentation_label_source', full_name='rv.protos.LabelSourceConfig.semantic_segmentation_label_source', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE, _LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE, _LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE, _LABELSOURCECONFIG_OBJECTDETECTIONGEOJSONSOURCE, _LABELSOURCECONFIG_CHIPCLASSIFICATIONGEOJSONSOURCE, _LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='label_source_config', full_name='rv.protos.LabelSourceConfig.label_source_config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=204,
  serialized_end=1791,
)

_LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE.fields_by_name['vector_source'].message_type = rastervision_dot_protos_dot_vector__source__pb2._VECTORSOURCECONFIG
_LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE.fields_by_name['vector_source'].message_type = rastervision_dot_protos_dot_vector__source__pb2._VECTORSOURCECONFIG
_LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE.fields_by_name['source'].message_type = rastervision_dot_protos_dot_raster__source__pb2._RASTERSOURCECONFIG
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE.fields_by_name['rgb_class_items'].message_type = rastervision_dot_protos_dot_class__item__pb2._CLASSITEM
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG_OBJECTDETECTIONGEOJSONSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG_CHIPCLASSIFICATIONGEOJSONSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE.fields_by_name['source'].message_type = rastervision_dot_protos_dot_raster__source__pb2._RASTERSOURCECONFIG
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE.fields_by_name['rgb_class_items'].message_type = rastervision_dot_protos_dot_class__item__pb2._CLASSITEM
_LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE.containing_type = _LABELSOURCECONFIG
_LABELSOURCECONFIG.fields_by_name['object_detection_geojson_source'].message_type = _LABELSOURCECONFIG_OBJECTDETECTIONGEOJSONSOURCE
_LABELSOURCECONFIG.fields_by_name['chip_classification_geojson_source'].message_type = _LABELSOURCECONFIG_CHIPCLASSIFICATIONGEOJSONSOURCE
_LABELSOURCECONFIG.fields_by_name['semantic_segmentation_raster_source'].message_type = _LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE
_LABELSOURCECONFIG.fields_by_name['custom_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_LABELSOURCECONFIG.fields_by_name['object_detection_label_source'].message_type = _LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE
_LABELSOURCECONFIG.fields_by_name['chip_classification_label_source'].message_type = _LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE
_LABELSOURCECONFIG.fields_by_name['semantic_segmentation_label_source'].message_type = _LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['object_detection_geojson_source'])
_LABELSOURCECONFIG.fields_by_name['object_detection_geojson_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['chip_classification_geojson_source'])
_LABELSOURCECONFIG.fields_by_name['chip_classification_geojson_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['semantic_segmentation_raster_source'])
_LABELSOURCECONFIG.fields_by_name['semantic_segmentation_raster_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['custom_config'])
_LABELSOURCECONFIG.fields_by_name['custom_config'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['object_detection_label_source'])
_LABELSOURCECONFIG.fields_by_name['object_detection_label_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['chip_classification_label_source'])
_LABELSOURCECONFIG.fields_by_name['chip_classification_label_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
_LABELSOURCECONFIG.oneofs_by_name['label_source_config'].fields.append(
  _LABELSOURCECONFIG.fields_by_name['semantic_segmentation_label_source'])
_LABELSOURCECONFIG.fields_by_name['semantic_segmentation_label_source'].containing_oneof = _LABELSOURCECONFIG.oneofs_by_name['label_source_config']
DESCRIPTOR.message_types_by_name['LabelSourceConfig'] = _LABELSOURCECONFIG

LabelSourceConfig = _reflection.GeneratedProtocolMessageType('LabelSourceConfig', (_message.Message,), dict(

  ObjectDetectionLabelSource = _reflection.GeneratedProtocolMessageType('ObjectDetectionLabelSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_OBJECTDETECTIONLABELSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.ObjectDetectionLabelSource)
    ))
  ,

  ChipClassificationLabelSource = _reflection.GeneratedProtocolMessageType('ChipClassificationLabelSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_CHIPCLASSIFICATIONLABELSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.ChipClassificationLabelSource)
    ))
  ,

  SemanticSegmentationLabelSource = _reflection.GeneratedProtocolMessageType('SemanticSegmentationLabelSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_SEMANTICSEGMENTATIONLABELSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.SemanticSegmentationLabelSource)
    ))
  ,

  ObjectDetectionGeoJSONSource = _reflection.GeneratedProtocolMessageType('ObjectDetectionGeoJSONSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_OBJECTDETECTIONGEOJSONSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.ObjectDetectionGeoJSONSource)
    ))
  ,

  ChipClassificationGeoJSONSource = _reflection.GeneratedProtocolMessageType('ChipClassificationGeoJSONSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_CHIPCLASSIFICATIONGEOJSONSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.ChipClassificationGeoJSONSource)
    ))
  ,

  SemanticSegmentationRasterSource = _reflection.GeneratedProtocolMessageType('SemanticSegmentationRasterSource', (_message.Message,), dict(
    DESCRIPTOR = _LABELSOURCECONFIG_SEMANTICSEGMENTATIONRASTERSOURCE,
    __module__ = 'rastervision.protos.label_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig.SemanticSegmentationRasterSource)
    ))
  ,
  DESCRIPTOR = _LABELSOURCECONFIG,
  __module__ = 'rastervision.protos.label_source_pb2'
  # @@protoc_insertion_point(class_scope:rv.protos.LabelSourceConfig)
  ))
_sym_db.RegisterMessage(LabelSourceConfig)
_sym_db.RegisterMessage(LabelSourceConfig.ObjectDetectionLabelSource)
_sym_db.RegisterMessage(LabelSourceConfig.ChipClassificationLabelSource)
_sym_db.RegisterMessage(LabelSourceConfig.SemanticSegmentationLabelSource)
_sym_db.RegisterMessage(LabelSourceConfig.ObjectDetectionGeoJSONSource)
_sym_db.RegisterMessage(LabelSourceConfig.ChipClassificationGeoJSONSource)
_sym_db.RegisterMessage(LabelSourceConfig.SemanticSegmentationRasterSource)


# @@protoc_insertion_point(module_scope)



# flake8: noqa

import rastervision as rv

ANALYZE = 'ANALYZE'
CHIP = 'CHIP'
TRAIN = 'TRAIN'
PREDICT = 'PREDICT'
EVAL = 'EVAL'
BUNDLE = 'BUNDLE'

from .command_config import CommandConfig
from .aux_command import (AuxCommand, AuxCommandOptions)
from .aux.api import *


def all_commands():
    return rv._registry.get_commands()



from rastervision.command import Command


class AuxCommandOptions:
    def __init__(self,
                 split_on=None,
                 inputs=lambda config: None,
                 outputs=lambda config: None,
                 include_by_default=False,
                 required_fields=None):
        """Instantiate an AuxCommandOptions object.

        Args:
            split_on (str): The property of the configuration to use when splitting.
            The configuration at this property must be a list.

            inputs: A function that, given the configuration, returns a list of
            URIs that are inputs into the command. Along with outputs, this allows
            Raster Vision to correctly determine if there are any missing inputs, or
            if the command has already been run. It will also allow the command to
            be run in the right sequence if run with other commands that will produce
            this command's inputs as their outputs.

            outputs: A function that, given the configuration, returns a list of
            URIs that are outputs of the command. See the details on inputs.

            include_by_default: Set this to True if you want this command to run
            by default, meaning it will run every time no specific commands are issued
            on the command line (e.g. how a standard command would run).

            required_fields: Set this to properties of the configuration that are
            required. If the user of the command does not set values into those
            configuration properties, an error will be thrown at configuration building
            time.

        """
        self.split_on = split_on
        self.inputs = inputs
        self.outputs = outputs
        self.include_by_default = include_by_default
        self.required_fields = required_fields


class AuxCommand(Command):
    """An abstract class representing an auxiliary command.
    The purpose of Aux commands is to make it easy to add
    custom functionality to Raster Vision.
    """

    command_type = None
    options = None

    def __init__(self, command_config):
        self.command_config = command_config



import pyproj

from rasterio.transform import (rowcol, xy)

from rastervision.data.crs_transformer import (CRSTransformer,
                                               IdentityCRSTransformer)


class RasterioCRSTransformer(CRSTransformer):
    """Transformer for a RasterioRasterSource."""

    def __init__(self, transform, image_crs, map_crs='epsg:4326'):
        """Construct transformer.

        Args:
            image_dataset: Rasterio DatasetReader
            map_crs: CRS code
        """
        self.map_proj = pyproj.Proj(init=map_crs)
        self.image_proj = pyproj.Proj(image_crs)

        super().__init__(image_crs, map_crs, transform)

    def map_to_pixel(self, map_point):
        """Transform point from map to pixel-based coordinates.

        Args:
            map_point: (x, y) tuple in map coordinates

        Returns:
            (x, y) tuple in pixel coordinates
        """
        image_point = pyproj.transform(self.map_proj, self.image_proj,
                                       map_point[0], map_point[1])
        pixel_point = rowcol(self.transform, image_point[0], image_point[1])
        pixel_point = (pixel_point[1], pixel_point[0])
        return pixel_point

    def pixel_to_map(self, pixel_point):
        """Transform point from pixel to map-based coordinates.

        Args:
            pixel_point: (x, y) tuple in pixel coordinates

        Returns:
            (x, y) tuple in map coordinates
        """
        image_point = xy(self.transform, int(pixel_point[1]),
                         int(pixel_point[0]))
        map_point = pyproj.transform(self.image_proj, self.map_proj,
                                     image_point[0], image_point[1])
        return map_point

    @classmethod
    def from_dataset(cls, dataset, map_crs='epsg:4326'):
        if dataset.crs is None:
            return IdentityCRSTransformer()
        transform = dataset.transform
        image_crs = dataset.crs
        return cls(transform, image_crs, map_crs)

    def get_affine_transform(self):
        return self.transform



from abc import abstractmethod
from copy import deepcopy
import logging

import rastervision as rv
from rastervision.core.config import (Config, ConfigBuilder,
                                      BundledConfigMixin)
from rastervision.data import (RasterTransformerConfig, StatsTransformerConfig)
from rastervision.protos.raster_source_pb2 \
    import RasterSourceConfig as RasterSourceConfigMsg

log = logging.getLogger(__name__)


class RasterSourceConfig(BundledConfigMixin, Config):
    deprecation_warnings = []

    def __init__(self, source_type, transformers=None, channel_order=None):
        if transformers is None:
            transformers = []

        self.source_type = source_type
        self.transformers = transformers
        self.channel_order = channel_order

    def to_proto(self):
        transformers = list(map(lambda c: c.to_proto(), self.transformers))
        msg = RasterSourceConfigMsg(
            source_type=self.source_type,
            channel_order=self.channel_order,
            transformers=transformers)
        return msg

    def save_bundle_files(self, bundle_dir):
        new_transformers = []
        files = []
        for transformer in self.transformers:
            new_transformer, t_files = transformer.save_bundle_files(
                bundle_dir)
            new_transformers.append(new_transformer)
            files.extend(t_files)

        new_config = self.to_builder() \
                         .with_transformers(new_transformers) \
                         .build()
        return (new_config, files)

    def load_bundle_files(self, bundle_dir):
        new_transformers = []
        for transformer in self.transformers:
            new_transformer = transformer.load_bundle_files(bundle_dir)
            new_transformers.append(new_transformer)
        return self.to_builder() \
                   .with_transformers(new_transformers) \
                   .build()

    @abstractmethod
    def create_source(self, tmp_dir, crs_transformer, extent, class_map):
        """Create the Raster Source for this configuration.
        """
        pass

    def to_builder(self):
        return rv._registry.get_config_builder(rv.RASTER_SOURCE,
                                               self.source_type)(self)

    @staticmethod
    def check_deprecation(source_type):
        # If source_type is deprecated and warning hasn't been shown yet, then warn.
        if (source_type in rv.raster_source_deprecated_map and
                source_type not in RasterSourceConfig.deprecation_warnings):
            RasterSourceConfig.deprecation_warnings.append(source_type)
            new_source_type = rv.raster_source_deprecated_map[source_type]
            log.warn(
                'RasterSource {} is deprecated. Please use {} instead.'.format(
                    source_type, new_source_type))

    def builder(source_type):
        RasterSourceConfig.check_deprecation(source_type)
        return rv._registry.get_config_builder(rv.RASTER_SOURCE, source_type)()

    @staticmethod
    def from_proto(msg):
        """Creates a TaskConfig from the specificed protobuf message
        """
        return rv._registry.get_config_builder(rv.RASTER_SOURCE, msg.source_type)() \
                           .from_proto(msg) \
                           .build()

    @abstractmethod
    def for_prediction(self, image_uri):
        """Creates a new config with the image_uri."""
        pass

    @abstractmethod
    def create_local(self, tmp_dir):
        """Returns a new config with a local copy of the image data
        if this image is remote.
        """
        pass

    def create_transformers(self):
        return list(map(lambda c: c.create_transformer(), self.transformers))

    def update_for_command(self, command_type, experiment_config,
                           context=None):
        for transformer in self.transformers:
            transformer.update_for_command(command_type, experiment_config,
                                           context)

    def report_io(self, command_type, io_def):
        for transformer in self.transformers:
            transformer.report_io(command_type, io_def)


class RasterSourceConfigBuilder(ConfigBuilder):
    def from_proto(self, msg):
        transformers = list(
            map(lambda m: RasterTransformerConfig.from_proto(m),
                msg.transformers))

        channel_order = list(msg.channel_order)
        if len(channel_order) == 0:
            channel_order = None
        return self.with_channel_order(channel_order) \
                   .with_transformers(transformers)

    def with_channel_order(self, channel_order):
        """Defines the channel order for this raster source.

        This defines the subset of channel indices and their order to use when extracting
        chips from raw imagery.

        Args:
            channel_order: list of channel indices
        """
        b = deepcopy(self)
        b.config['channel_order'] = channel_order
        return b

    def with_transformers(self, transformers):
        """Transformers to be applied to the raster data.

            Args:
                transformers: A list of transformers to apply to the
                    raster data.

        """
        b = deepcopy(self)
        b.config['transformers'] = list(transformers)
        return b

    def with_transformer(self, transformer):
        """A transformer to be applied to the raster data.

            Args:
                transformer: A transformer to apply to the raster
                    data.

        """
        return self.with_transformers([transformer])

    def with_stats_transformer(self):
        """Add a stats transformer to the raster source."""
        b = deepcopy(self)
        transformers = b.config.get('transformers')
        if transformers:
            b.config['transformers'] = transformers.append(
                StatsTransformerConfig())
        else:
            b.config['transformers'] = [StatsTransformerConfig()]
        return b



class Dataset:
    def __init__(self,
                 train_scenes=[],
                 validation_scenes=[],
                 test_scenes=[],
                 augmentors=[]):
        self.train_scenes = train_scenes
        self.validation_scenes = validation_scenes
        self.test_scenes = test_scenes
        self.augmentors = augmentors



# flake8: noqa

from rastervision.command.command import *
from rastervision.command.command_config import *
from rastervision.command.analyze_command import *
from rastervision.command.analyze_command_config import *
from rastervision.command.chip_command import *
from rastervision.command.chip_command_config import *
from rastervision.command.train_command import *
from rastervision.command.train_command_config import *
from rastervision.command.predict_command import *
from rastervision.command.predict_command_config import *
from rastervision.command.eval_command import *
from rastervision.command.eval_command_config import *
from rastervision.command.bundle_command import *
from rastervision.command.bundle_command_config import *



# flake8: noqa
# -*- coding: utf-8 -*-
"""ResNet50 model for Keras.
# Reference:
- [Deep Residual Learning for Image Recognition](https://arxiv.org/abs/1512.03385)
Adapted from code contributed by BigMoyan.
Adapted from Keras.
"""
from __future__ import print_function
from __future__ import absolute_import

import warnings

from keras.layers import Input
from keras import layers
from keras.layers import Dense
from keras.layers import Activation
from keras.layers import Flatten
from keras.layers import Conv2D
from keras.layers import MaxPooling2D
from keras.layers import ZeroPadding2D
from keras.layers import AveragePooling2D
from keras.layers import GlobalAveragePooling2D
from keras.layers import GlobalMaxPooling2D
from keras.layers import BatchNormalization
from keras.models import Model
from keras import backend as K
from keras.engine.topology import get_source_inputs
from keras.utils import layer_utils
from keras.utils.data_utils import get_file
from keras.applications.imagenet_utils import (decode_predictions,
                                               preprocess_input)


def identity_block(input_tensor, kernel_size, filters, stage, block):
    """The identity block is the block that has no conv layer at shortcut.
    # Arguments
        input_tensor: input tensor
        kernel_size: defualt 3, the kernel size of middle conv layer at main path
        filters: list of integers, the filterss of 3 conv layer at main path
        stage: integer, current stage label, used for generating layer names
        block: 'a','b'..., current block label, used for generating layer names
    # Returns
        Output tensor for the block.
    """
    filters1, filters2, filters3 = filters
    if K.image_data_format() == 'channels_last':
        bn_axis = 3
    else:
        bn_axis = 1
    conv_name_base = 'res' + str(stage) + block + '_branch'
    bn_name_base = 'bn' + str(stage) + block + '_branch'
    act_name = 'act' + str(stage) + block

    x = Conv2D(filters1, (1, 1), name=conv_name_base + '2a')(input_tensor)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2a')(x)
    x = Activation('relu')(x)

    x = Conv2D(
        filters2, kernel_size, padding='same', name=conv_name_base + '2b')(x)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2b')(x)
    x = Activation('relu')(x)

    x = Conv2D(filters3, (1, 1), name=conv_name_base + '2c')(x)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2c')(x)

    x = layers.add([x, input_tensor])
    x = Activation('relu', name=act_name)(x)
    return x


def conv_block(input_tensor,
               kernel_size,
               filters,
               stage,
               block,
               strides=(2, 2)):
    """conv_block is the block that has a conv layer at shortcut
    # Arguments
        input_tensor: input tensor
        kernel_size: defualt 3, the kernel size of middle conv layer at main path
        filters: list of integers, the filterss of 3 conv layer at main path
        stage: integer, current stage label, used for generating layer names
        block: 'a','b'..., current block label, used for generating layer names
    # Returns
        Output tensor for the block.
    Note that from stage 3, the first conv layer at main path is with strides=(2,2)
    And the shortcut should have strides=(2,2) as well
    """
    filters1, filters2, filters3 = filters
    if K.image_data_format() == 'channels_last':
        bn_axis = 3
    else:
        bn_axis = 1
    conv_name_base = 'res' + str(stage) + block + '_branch'
    bn_name_base = 'bn' + str(stage) + block + '_branch'
    act_name = 'act' + str(stage) + block

    x = Conv2D(
        filters1, (1, 1), strides=strides,
        name=conv_name_base + '2a')(input_tensor)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2a')(x)
    x = Activation('relu')(x)

    x = Conv2D(
        filters2, kernel_size, padding='same', name=conv_name_base + '2b')(x)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2b')(x)
    x = Activation('relu')(x)

    x = Conv2D(filters3, (1, 1), name=conv_name_base + '2c')(x)
    x = BatchNormalization(axis=bn_axis, name=bn_name_base + '2c')(x)

    shortcut = Conv2D(
        filters3, (1, 1), strides=strides,
        name=conv_name_base + '1')(input_tensor)
    shortcut = BatchNormalization(
        axis=bn_axis, name=bn_name_base + '1')(shortcut)

    x = layers.add([x, shortcut])
    x = Activation('relu', name=act_name)(x)
    return x


def ResNet50(include_top=True,
             weights=None,
             load_weights_by_name=False,
             input_tensor=None,
             input_shape=None,
             pooling=None,
             classes=1000,
             activation='softmax'):
    """Instantiates the ResNet50 architecture.
    Optionally loads weights pre-trained
    on ImageNet.
    # Arguments
        include_top: whether to include the fully-connected
            layer at the top of the network.
        weights: one of `None` (random initialization)
            or a path to model weights.
        input_tensor: optional Keras tensor (i.e. output of `layers.Input()`)
            to use as image input for the model.
        input_shape: optional shape tuple, only to be specified
            if `include_top` is False (otherwise the input shape
            has to be `(224, 224, 3)` (with `channels_last` data format)
            or `(3, 224, 244)` (with `channels_first` data format).
            It should have exactly 3 inputs channels,
            and width and height should be no smaller than 197.
            E.g. `(200, 200, 3)` would be one valid value.
        pooling: Optional pooling mode for feature extraction
            when `include_top` is `False`.
            - `None` means that the output of the model will be
                the 4D tensor output of the
                last convolutional layer.
            - `avg` means that global average pooling
                will be applied to the output of the
                last convolutional layer, and thus
                the output of the model will be a 2D tensor.
            - `max` means that global max pooling will
                be applied.
        classes: optional number of classes to classify images
            into, only to be specified if `include_top` is True, and
            if no `weights` argument is specified.
        activation: activation function for top fully-connected layer
    # Returns
        A Keras model instance.
    # Raises
        ValueError: in case of invalid argument for `weights`,
            or invalid input shape.
    """
    if input_tensor is None:
        img_input = Input(shape=input_shape)
    else:
        img_input = input_tensor

    bn_axis = 3

    x = ZeroPadding2D((3, 3))(img_input)
    x = Conv2D(64, (7, 7), strides=(2, 2), name='conv1')(x)
    x = BatchNormalization(axis=bn_axis, name='bn_conv1')(x)
    x = Activation('relu')(x)
    x = MaxPooling2D((3, 3), strides=(2, 2))(x)

    x = conv_block(x, 3, [64, 64, 256], stage=2, block='a', strides=(1, 1))
    x = identity_block(x, 3, [64, 64, 256], stage=2, block='b')
    x = identity_block(x, 3, [64, 64, 256], stage=2, block='c')

    x = conv_block(x, 3, [128, 128, 512], stage=3, block='a')
    x = identity_block(x, 3, [128, 128, 512], stage=3, block='b')
    x = identity_block(x, 3, [128, 128, 512], stage=3, block='c')
    x = identity_block(x, 3, [128, 128, 512], stage=3, block='d')

    x = conv_block(x, 3, [256, 256, 1024], stage=4, block='a')
    x = identity_block(x, 3, [256, 256, 1024], stage=4, block='b')
    x = identity_block(x, 3, [256, 256, 1024], stage=4, block='c')
    x = identity_block(x, 3, [256, 256, 1024], stage=4, block='d')
    x = identity_block(x, 3, [256, 256, 1024], stage=4, block='e')
    x = identity_block(x, 3, [256, 256, 1024], stage=4, block='f')

    x = conv_block(x, 3, [512, 512, 2048], stage=5, block='a')
    x = identity_block(x, 3, [512, 512, 2048], stage=5, block='b')
    x = identity_block(x, 3, [512, 512, 2048], stage=5, block='c')

    x = AveragePooling2D((7, 7), name='avg_pool')(x)

    if include_top:
        x = Flatten()(x)
        x = Dense(classes, activation=activation, name='dense')(x)
    else:
        if pooling == 'avg':
            x = GlobalAveragePooling2D()(x)
        elif pooling == 'max':
            x = GlobalMaxPooling2D()(x)

    # Ensure that the model takes into account
    # any potential predecessors of `input_tensor`.
    if input_tensor is not None:
        inputs = get_source_inputs(input_tensor)
    else:
        inputs = img_input
    # Create model.
    model = Model(inputs, x, name='resnet50')

    # load weights
    if weights:
        model.load_weights(weights, by_name=load_weights_by_name)

    return model



class CRSTransformer(object):
    """Transforms map points in some CRS into pixel coordinates.

    Each transformer is associated with a particular RasterSource."""

    def __init__(self, image_crs=None, map_crs=None, transform=None):
        self.image_crs = image_crs
        self.map_crs = map_crs
        self.transform = transform

    def map_to_pixel(self, map_point):
        """Transform point from map to pixel-based coordinates.

        Args:
            map_point: (x, y) tuple in map coordinates (eg. lon/lat). x and y can be
            single values or array-like.

        Returns:
            (x, y) tuple in pixel coordinates
        """
        pass

    def pixel_to_map(self, pixel_point):
        """Transform point from pixel to map-based coordinates.

        Args:
            pixel_point: (x, y) tuple in pixel coordinates. x and y can be
            single values or array-like.

        Returns:
            (x, y) tuple in map coordinates (eg. lon/lat)
        """
        pass

    def get_image_crs(self):
        return self.image_crs

    def get_map_crs(self):
        return self.map_crs

    def get_affine_transform(self):
        return self.transform



from os.path import join, basename
import glob

from PIL import Image
import torchvision
from torch.utils.data import DataLoader, Dataset

from rastervision.backend.torch_utils.data import DataBunch


class ToTensor(object):
    def __init__(self):
        self.to_tensor = torchvision.transforms.ToTensor()

    def __call__(self, x, y):
        return (self.to_tensor(x), (255 * self.to_tensor(y)).squeeze().long())


class ComposeTransforms(object):
    def __init__(self, transforms):
        self.transforms = transforms

    def __call__(self, x, y):
        for t in self.transforms:
            x, y = t(x, y)
        return x, y


class SegmentationDataset(Dataset):
    def __init__(self, data_dir, transforms=None):
        self.data_dir = data_dir
        self.img_paths = glob.glob(join(data_dir, 'img', '*.png'))
        self.transforms = transforms

    def __getitem__(self, ind):
        img_path = self.img_paths[ind]
        label_path = join(self.data_dir, 'labels', basename(img_path))
        x = Image.open(img_path)
        y = Image.open(label_path)

        if self.transforms is not None:
            x, y = self.transforms(x, y)
        return (x, y)

    def __len__(self):
        return len(self.img_paths)


def build_databunch(data_dir, img_sz, batch_sz, class_names):
    # set to zero to prevent "dataloader is killed by signal"
    # TODO fix this
    num_workers = 0

    train_dir = join(data_dir, 'train')
    valid_dir = join(data_dir, 'valid')

    aug_transforms = ComposeTransforms([ToTensor()])
    transforms = ComposeTransforms([ToTensor()])

    train_ds = SegmentationDataset(train_dir, transforms=aug_transforms)
    valid_ds = SegmentationDataset(valid_dir, transforms=transforms)

    train_dl = DataLoader(
        train_ds,
        shuffle=True,
        batch_size=batch_sz,
        num_workers=num_workers,
        drop_last=True,
        pin_memory=True)
    valid_dl = DataLoader(
        valid_ds,
        batch_size=batch_sz,
        num_workers=num_workers,
        pin_memory=True)

    return DataBunch(train_ds, train_dl, valid_ds, valid_dl, class_names)



from copy import deepcopy

import rastervision as rv
from rastervision.data.vector_source.vector_source_config import (
    VectorSourceConfig, VectorSourceConfigBuilder)
from rastervision.data.vector_source.geojson_vector_source import GeoJSONVectorSource
from rastervision.data.vector_source.class_inference import ClassInferenceOptions


class GeoJSONVectorSourceConfig(VectorSourceConfig):
    def __init__(self,
                 uri,
                 class_id_to_filter=None,
                 default_class_id=1,
                 line_bufs=None,
                 point_bufs=None):
        self.uri = uri
        super().__init__(
            rv.GEOJSON_SOURCE,
            class_id_to_filter=class_id_to_filter,
            default_class_id=default_class_id,
            line_bufs=line_bufs,
            point_bufs=point_bufs)

    def to_proto(self):
        msg = super().to_proto()
        msg.geojson.uri = self.uri
        return msg

    def create_source(self, crs_transformer=None, extent=None, class_map=None):
        return GeoJSONVectorSource(
            self.uri,
            crs_transformer,
            line_bufs=self.line_bufs,
            point_bufs=self.point_bufs,
            class_inf_opts=ClassInferenceOptions(
                class_map=class_map,
                class_id_to_filter=self.class_id_to_filter,
                default_class_id=self.default_class_id))

    def report_io(self, command_type, io_def):
        io_def.add_input(self.uri)


class GeoJSONVectorSourceConfigBuilder(VectorSourceConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'uri': prev.uri,
                'class_id_to_filter': prev.class_id_to_filter,
                'default_class_id': prev.default_class_id,
                'line_bufs': prev.line_bufs,
                'point_bufs': prev.point_bufs
            }

        super().__init__(GeoJSONVectorSourceConfig, config)

    def validate(self):
        if self.config.get('uri') is None:
            raise rv.ConfigError(
                'GeoJSONVectorSourceConfigBuilder requires uri which '
                'can be set using "with_uri".')

        super().validate()

    def from_proto(self, msg):
        b = super().from_proto(msg)
        b = b.with_uri(msg.geojson.uri)
        return b

    def with_uri(self, uri):
        b = deepcopy(self)
        b.config['uri'] = uri
        return b



from abc import (ABC, abstractmethod)


class Augmentor(ABC):
    """Defines a method for augmenting training data.
    """

    @abstractmethod
    def process(self, training_data, tmp_dir):
        """Augment training data.

        Args:
           training_data: The TrainingData to augment

        Returns:
           augmented TrainingData
        """
        pass



import numpy as np
from shapely.geometry import shape

from rastervision.core.box import Box
from rastervision.data.label import Labels
from rastervision.data.label.tfod_utils.np_box_list import BoxList
from rastervision.data.label.tfod_utils.np_box_list_ops import (
    prune_non_overlapping_boxes, clip_to_window, concatenate,
    non_max_suppression)


class ObjectDetectionLabels(Labels):
    """A set of boxes and associated class_ids and scores.

    Implemented using the Tensorflow Object Detection API's BoxList class.
    """

    def __init__(self, npboxes, class_ids, scores=None):
        """Construct a set of object detection labels.

        Args:
            npboxes: float numpy array of size nx4 with cols
                ymin, xmin, ymax, xmax. Should be in pixel coordinates within
                the global frame of reference.
            class_ids: int numpy array of size n with class ids starting at 1
            scores: float numpy array of size n
        """
        self.boxlist = BoxList(npboxes)
        # This field name actually needs to be 'classes' to be able to use
        # certain utility functions in the TF Object Detection API.
        self.boxlist.add_field('classes', class_ids)
        # We need to ensure that there is always a scores field so that the
        # concatenate method will work with empty labels objects.
        if scores is None:
            scores = np.ones(class_ids.shape)
        self.boxlist.add_field('scores', scores)

    def __add__(self, other):
        return ObjectDetectionLabels.concatenate(self, other)

    def __eq__(self, other):
        return (isinstance(other, ObjectDetectionLabels)
                and self.to_dict() == other.to_dict())

    def assert_equal(self, expected_labels):
        np.testing.assert_array_equal(self.get_npboxes(),
                                      expected_labels.get_npboxes())
        np.testing.assert_array_equal(self.get_class_ids(),
                                      expected_labels.get_class_ids())
        np.testing.assert_array_equal(self.get_scores(),
                                      expected_labels.get_scores())

    def filter_by_aoi(self, aoi_polygons):
        boxes = self.get_boxes()
        class_ids = self.get_class_ids()
        scores = self.get_scores()

        new_boxes = []
        new_class_ids = []
        new_scores = []
        for box, class_id, score in zip(boxes, class_ids, scores):
            box_poly = box.to_shapely()
            for aoi in aoi_polygons:
                if box_poly.within(aoi):
                    new_boxes.append(box.npbox_format())
                    new_class_ids.append(class_id)
                    new_scores.append(score)
                    break

        if len(new_boxes) == 0:
            return ObjectDetectionLabels.make_empty()

        return ObjectDetectionLabels(
            np.array(new_boxes), np.array(new_class_ids), np.array(new_scores))

    @staticmethod
    def make_empty():
        npboxes = np.empty((0, 4))
        class_ids = np.empty((0, ))
        scores = np.empty((0, ))
        return ObjectDetectionLabels(npboxes, class_ids, scores)

    @staticmethod
    def from_boxlist(boxlist):
        """Make ObjectDetectionLabels from BoxList object."""
        scores = (boxlist.get_field('scores')
                  if boxlist.has_field('scores') else None)
        return ObjectDetectionLabels(
            boxlist.get(), boxlist.get_field('classes'), scores=scores)

    @staticmethod
    def from_geojson(geojson, extent=None):
        """Convert GeoJSON to ObjectDetectionLabels object.

        If extent is provided, filter out the boxes that lie "more than a little
        bit" outside the extent.

        Args:
            geojson: (dict) normalized GeoJSON (see VectorSource)
            extent: (Box) in pixel coords

        Returns:
            ObjectDetectionLabels
        """
        boxes = []
        class_ids = []
        scores = []

        for f in geojson['features']:
            geom = shape(f['geometry'])
            (xmin, ymin, xmax, ymax) = geom.bounds
            boxes.append(Box(ymin, xmin, ymax, xmax))

            props = f['properties']
            class_ids.append(props['class_id'])
            scores.append(props.get('score', 1.0))

        if len(boxes):
            boxes = np.array(
                [box.npbox_format() for box in boxes], dtype=float)
            class_ids = np.array(class_ids)
            scores = np.array(scores)
            labels = ObjectDetectionLabels(boxes, class_ids, scores=scores)
        else:
            labels = ObjectDetectionLabels.make_empty()

        if extent is not None:
            labels = ObjectDetectionLabels.get_overlapping(
                labels, extent, ioa_thresh=0.8, clip=True)
        return labels

    def get_boxes(self):
        """Return list of Boxes."""
        return [Box.from_npbox(npbox) for npbox in self.boxlist.get()]

    def get_npboxes(self):
        return self.boxlist.get()

    def get_scores(self):
        if self.boxlist.has_field('scores'):
            return self.boxlist.get_field('scores')
        return None

    def get_class_ids(self):
        return self.boxlist.get_field('classes')

    def __len__(self):
        return self.boxlist.get().shape[0]

    def __str__(self):
        return str(self.boxlist.get())

    def to_boxlist(self):
        return self.boxlist

    def to_dict(self):
        """Returns a dict version of these labels.

        The Dict has a Box as a key, and a tuple of (class_id, score)
        as the values.
        """
        d = {}
        boxes = list(map(Box.from_npbox, self.get_npboxes()))
        classes = list(self.get_class_ids())
        scores = list(self.get_scores())
        for box, class_id, score in zip(boxes, classes, scores):
            d[box.tuple_format()] = (class_id, score)
        return d

    @staticmethod
    def local_to_global(npboxes, window):
        """Convert from local to global coordinates.

        The local coordinates are row/col within the window frame of reference.
        The global coordinates are row/col within the extent of a RasterSource.
        """
        xmin = window.xmin
        ymin = window.ymin
        return npboxes + np.array([[ymin, xmin, ymin, xmin]])

    @staticmethod
    def global_to_local(npboxes, window):
        """Convert from global to local coordinates.

        The global coordinates are row/col within the extent of a RasterSource.
        The local coordinates are row/col within the window frame of reference.
        """
        xmin = window.xmin
        ymin = window.ymin
        return npboxes - np.array([[ymin, xmin, ymin, xmin]])

    @staticmethod
    def local_to_normalized(npboxes, window):
        """Convert from local to normalized coordinates.

        The local coordinates are row/col within the window frame of reference.
        Normalized coordinates range from 0 to 1 on each (height/width) axis.
        """
        height = window.get_height()
        width = window.get_width()
        return npboxes / np.array([[height, width, height, width]])

    @staticmethod
    def normalized_to_local(npboxes, window):
        """Convert from normalized to local coordinates.

        Normalized coordinates range from 0 to 1 on each (height/width) axis.
        The local coordinates are row/col within the window frame of reference.
        """
        height = window.get_height()
        width = window.get_width()
        return npboxes * np.array([[height, width, height, width]])

    @staticmethod
    def get_overlapping(labels, window, ioa_thresh=0.000001, clip=False):
        """Return subset of labels that overlap with window.

        Args:
            labels: ObjectDetectionLabels
            window: Box
            ioa_thresh: the minimum IOA for a box to be considered as
                overlapping
            clip: if True, clip label boxes to the window
        """
        window_npbox = window.npbox_format()
        window_boxlist = BoxList(np.expand_dims(window_npbox, axis=0))
        boxlist = prune_non_overlapping_boxes(
            labels.boxlist, window_boxlist, minoverlap=ioa_thresh)
        if clip:
            boxlist = clip_to_window(boxlist, window_npbox)

        return ObjectDetectionLabels.from_boxlist(boxlist)

    @staticmethod
    def concatenate(labels1, labels2):
        """Return concatenation of labels.

        Args:
            labels1: ObjectDetectionLabels
            labels2: ObjectDetectionLabels
        """
        new_boxlist = concatenate([labels1.to_boxlist(), labels2.to_boxlist()])
        return ObjectDetectionLabels.from_boxlist(new_boxlist)

    @staticmethod
    def prune_duplicates(labels, score_thresh, merge_thresh):
        """Remove duplicate boxes.

        Runs non-maximum suppression to remove duplicate boxes that result from
        sliding window prediction algorithm.

        Args:
            labels: ObjectDetectionLabels
            score_thresh: the minimum allowed score of boxes
            merge_thresh: the minimum IOA allowed when merging two boxes
                together

        Returns:
            ObjectDetectionLabels
        """
        max_output_size = 1000000
        pruned_boxlist = non_max_suppression(
            labels.boxlist,
            max_output_size=max_output_size,
            iou_threshold=merge_thresh,
            score_threshold=score_thresh)
        return ObjectDetectionLabels.from_boxlist(pruned_boxlist)



from rastervision.core.box import Box
from rastervision.data.label import Labels


class ChipClassificationLabels(Labels):
    """Represents a spatial grid of cells associated with classes."""

    def __init__(self):
        self.cell_to_class_id = {}

    def __len__(self):
        return len(self.cell_to_class_id)

    def __eq__(self, other):
        return (isinstance(other, ChipClassificationLabels)
                and self.cell_to_class_id == other.cell_to_class_id)

    def __add__(self, other):
        result = ChipClassificationLabels()
        result.extend(self)
        result.extend(other)
        return result

    def filter_by_aoi(self, aoi_polygons):
        result = ChipClassificationLabels()
        for cell in self.cell_to_class_id:
            cell_box = Box.from_tuple(cell)
            cell_poly = cell_box.to_shapely()
            for aoi in aoi_polygons:
                if cell_poly.within(aoi):
                    (class_id, scores) = self.cell_to_class_id[cell]
                    result.set_cell(cell_box, class_id, scores)
        return result

    def set_cell(self, cell, class_id, scores=None):
        """Set cell and its class_id.

        Args:
            cell: (Box)
            class_id: int
            scores: 1d numpy array of probabilities for each class
        """
        if scores is not None:
            scores = list(map(lambda x: float(x), list(scores)))
        self.cell_to_class_id[cell.tuple_format()] = (class_id, scores)

    def get_cell_class_id(self, cell):
        """Return class_id for a cell.

        Args:
            cell: (Box)
        """
        result = self.cell_to_class_id.get(cell.tuple_format())
        if result:
            return result[0]
        else:
            return None

    def get_cell_scores(self, cell):
        """Return scores for a cell.

        Args:
            cell: (Box)
        """
        result = self.cell_to_class_id.get(cell.tuple_format())
        if result:
            return result[1]
        else:
            return None

    def get_cell_values(self, cell):
        """Return a tuple of (class_id, scores) for a cell.

        Args:
            cell: (Box)
        """
        return self.cell_to_class_id.get(cell.tuple_format())

    def get_singleton_labels(self, cell):
        """Return Labels object representing a single cell.

        Args:
            cell: (Box)
        """
        class_id, scores = self.get_cell_values(cell)
        labels = ChipClassificationLabels()
        labels.set_cell(cell, class_id, scores)
        return labels

    def get_cells(self):
        """Return list of all cells (list of Box)."""
        return [
            Box.from_npbox(box_tup)
            for box_tup in self.cell_to_class_id.keys()
        ]

    def get_class_ids(self):
        """Return list of class_ids for all cells."""
        return list(map(lambda x: x[0], self.cell_to_class_id.values()))

    def get_scores(self):
        """Return list of scores for all cells."""
        return list(map(lambda x: x[1], self.cell_to_class_id.values()))

    def get_values(self):
        """Return list of class_ids and scores for all cells."""
        return list(self.cell_to_class_id.values())

    def extend(self, labels):
        """Adds cells contained in labels.

        Args:
            labels: ChipClassificationLabels
        """
        for cell in labels.get_cells():
            class_id, scores = labels.get_cell_values(cell)
            self.set_cell(cell, class_id, scores)



from abc import abstractmethod
import logging

import rastervision as rv
from rastervision.core.config import (Config, ConfigBuilder)
from rastervision.protos.label_source_pb2 import LabelSourceConfig as LabelSourceConfigMsg

log = logging.getLogger(__name__)


class LabelSourceConfig(Config):
    deprecation_warnings = []

    def __init__(self, source_type):
        self.source_type = source_type

    def to_proto(self):
        msg = LabelSourceConfigMsg()
        msg.source_type = self.source_type
        return msg

    @abstractmethod
    def create_source(self, task_config, extent, crs_transformer, tmp_dir):
        """Create the Label Source for this configuration.

           Args:
              task_config: The TaskConfig for which this label source is supplying labels.
              extent: extent that this label source is applicable for
              crs_transformer: The crs_transformer used by the raster source this
                               label source is describing.
              tmp_dir: The temporary directory to use if files will need to be downloaded,
                       or None if only using local files.
        """
        pass

    def to_builder(self):
        return rv._registry.get_config_builder(rv.LABEL_SOURCE,
                                               self.source_type)(self)

    @staticmethod
    def check_deprecation(source_type):
        # If source_type is deprecated and warning hasn't been shown yet, then warn.
        if (source_type in rv.label_source_deprecated_map
                and source_type not in LabelSourceConfig.deprecation_warnings):
            LabelSourceConfig.deprecation_warnings.append(source_type)
            new_source_type = rv.label_source_deprecated_map[source_type]
            log.warn(
                'LabelSource {} is deprecated. Please use {} instead.'.format(
                    source_type, new_source_type))

    @staticmethod
    def builder(source_type):
        LabelSourceConfig.check_deprecation(source_type)
        return rv._registry.get_config_builder(rv.LABEL_SOURCE, source_type)()

    @staticmethod
    def from_proto(msg):
        """Creates a config from the specificed protobuf message
        """
        return rv._registry.get_config_builder(rv.LABEL_SOURCE, msg.source_type)() \
                           .from_proto(msg) \
                           .build()


class LabelSourceConfigBuilder(ConfigBuilder):
    pass



# flake8: noqa

from rastervision.analyzer.analyzer import *
from rastervision.analyzer.analyzer_config import *
from rastervision.analyzer.stats_analyzer import *
from rastervision.analyzer.stats_analyzer_config import *



from abc import ABC, abstractmethod

from rastervision.rv_config import RVConfig


class Command(ABC):
    @abstractmethod
    def run(self, tmp_dir):
        """Run the command."""
        pass

    def set_tmp_dir(self, tmp_dir):
        self._tmp_dir = tmp_dir

    def get_tmp_dir(self):
        if hasattr(self, '_tmp_dir') and self._tmp_dir:
            if isinstance(self._tmp_dir, str):
                return self._tmp_dir
            else:
                return self._tmp_dir.name
        else:
            tmp_dir = RVConfig.get_tmp_dir()
            self.set_tmp_dir(tmp_dir)
            return tmp_dir.name


class NoOpCommand(Command):
    """Defines a command that does nothing.
    """

    def run(self, tmp_dir):
        pass



import rastervision as rv
import rastervision.filesystem as rvfs
from rastervision.cli import Verbosity
from rastervision.rv_config import RVConfig
from rastervision.plugin import PluginRegistry
from rastervision.data.raster_source.default import (
    RasterioSourceDefaultProvider)
from rastervision.data.vector_source.default import (
    GeoJSONVectorSourceDefaultProvider, VectorTileVectorSourceDefaultProvider)
from rastervision.data.label_source.default import (
    ObjectDetectionLabelSourceDefaultProvider,
    ChipClassificationLabelSourceDefaultProvider,
    SemanticSegmentationLabelSourceDefaultProvider)
from rastervision.data.label_store.default import (
    ObjectDetectionGeoJSONStoreDefaultProvider,
    ChipClassificationGeoJSONStoreDefaultProvider,
    SemanticSegmentationRasterStoreDefaultProvider)
from rastervision.evaluation.default import (
    ObjectDetectionEvaluatorDefaultProvider,
    ChipClassificationEvaluatorDefaultProvider,
    SemanticSegmentationEvaluatorDefaultProvider)


class RegistryError(Exception):
    pass


class Registry:
    """Singleton that holds instances of Raster Vision types,
       to be referenced by configuration code by key.
    """

    def __init__(self):
        self._rv_config = None
        self._plugin_registry = None

        self._internal_config_builders = {
            # Tasks
            (rv.TASK, rv.OBJECT_DETECTION):
            rv.task.ObjectDetectionConfigBuilder,
            (rv.TASK, rv.CHIP_CLASSIFICATION):
            rv.task.ChipClassificationConfigBuilder,
            (rv.TASK, rv.SEMANTIC_SEGMENTATION):
            rv.task.SemanticSegmentationConfigBuilder,

            # Raster Transformers
            (rv.RASTER_TRANSFORMER, rv.STATS_TRANSFORMER):
            rv.data.StatsTransformerConfigBuilder,

            # Raster Sources
            (rv.RASTER_SOURCE, rv.RASTERIZED_SOURCE):
            rv.data.RasterizedSourceConfigBuilder,
            (rv.RASTER_SOURCE, rv.RASTERIO_SOURCE):
            rv.data.RasterioSourceConfigBuilder,

            # Alias provided for backwards compatibility.
            (rv.RASTER_SOURCE, rv.GEOJSON_SOURCE):
            rv.data.RasterizedSourceConfigBuilder,
            (rv.RASTER_SOURCE, rv.GEOTIFF_SOURCE):
            rv.data.RasterioSourceConfigBuilder,
            (rv.RASTER_SOURCE, rv.IMAGE_SOURCE):
            rv.data.RasterioSourceConfigBuilder,

            # Vector Sources
            (rv.VECTOR_SOURCE, rv.VECTOR_TILE_SOURCE):
            rv.data.VectorTileVectorSourceConfigBuilder,
            (rv.VECTOR_SOURCE, rv.GEOJSON_SOURCE):
            rv.data.GeoJSONVectorSourceConfigBuilder,

            # Label Sources
            (rv.LABEL_SOURCE, rv.CHIP_CLASSIFICATION):
            rv.data.ChipClassificationLabelSourceConfigBuilder,
            (rv.LABEL_SOURCE, rv.OBJECT_DETECTION):
            rv.data.ObjectDetectionLabelSourceConfigBuilder,
            (rv.LABEL_SOURCE, rv.SEMANTIC_SEGMENTATION):
            rv.data.SemanticSegmentationLabelSourceConfigBuilder,

            # Label Source aliases provided for backward-compatibility
            (rv.LABEL_SOURCE, rv.OBJECT_DETECTION_GEOJSON):
            rv.data.ObjectDetectionLabelSourceConfigBuilder,
            (rv.LABEL_SOURCE, rv.CHIP_CLASSIFICATION_GEOJSON):
            rv.data.ChipClassificationLabelSourceConfigBuilder,
            (rv.LABEL_SOURCE, rv.SEMANTIC_SEGMENTATION_RASTER):
            rv.data.SemanticSegmentationLabelSourceConfigBuilder,

            # Label Stores
            (rv.LABEL_STORE, rv.OBJECT_DETECTION_GEOJSON):
            rv.data.ObjectDetectionGeoJSONStoreConfigBuilder,
            (rv.LABEL_STORE, rv.CHIP_CLASSIFICATION_GEOJSON):
            rv.data.ChipClassificationGeoJSONStoreConfigBuilder,
            (rv.LABEL_STORE, rv.SEMANTIC_SEGMENTATION_RASTER):
            rv.data.SemanticSegmentationRasterStoreConfigBuilder,

            # Analyzers
            (rv.ANALYZER, rv.STATS_ANALYZER):
            rv.analyzer.StatsAnalyzerConfigBuilder,

            # Augmentors
            (rv.AUGMENTOR, rv.NODATA_AUGMENTOR):
            rv.augmentor.NodataAugmentorConfigBuilder,

            # Evaluators
            (rv.EVALUATOR, rv.CHIP_CLASSIFICATION_EVALUATOR):
            rv.evaluation.ChipClassificationEvaluatorConfigBuilder,
            (rv.EVALUATOR, rv.OBJECT_DETECTION_EVALUATOR):
            rv.evaluation.ObjectDetectionEvaluatorConfigBuilder,
            (rv.EVALUATOR, rv.SEMANTIC_SEGMENTATION_EVALUATOR):
            rv.evaluation.SemanticSegmentationEvaluatorConfigBuilder,
        }

        if rv.backend.tf_available:
            self._internal_config_builders[(rv.BACKEND, rv.TF_OBJECT_DETECTION)] = \
                rv.backend.TFObjectDetectionConfigBuilder
            self._internal_config_builders[(rv.BACKEND, rv.KERAS_CLASSIFICATION)] = \
                rv.backend.KerasClassificationConfigBuilder
            self._internal_config_builders[(rv.BACKEND, rv.TF_DEEPLAB)] = \
                rv.backend.TFDeeplabConfigBuilder

        if rv.backend.pytorch_available:
            key_val = (rv.BACKEND, rv.PYTORCH_CHIP_CLASSIFICATION)
            self._internal_config_builders[key_val] = \
                rv.backend.PyTorchChipClassificationConfigBuilder
            key_val = (rv.BACKEND, rv.PYTORCH_SEMANTIC_SEGMENTATION)
            self._internal_config_builders[key_val] = \
                rv.backend.PyTorchSemanticSegmentationConfigBuilder
            key_val = (rv.BACKEND, rv.PYTORCH_OBJECT_DETECTION)
            self._internal_config_builders[key_val] = \
                rv.backend.PyTorchObjectDetectionConfigBuilder

        self._internal_default_raster_sources = [RasterioSourceDefaultProvider]

        self._internal_default_vector_sources = [
            GeoJSONVectorSourceDefaultProvider,
            VectorTileVectorSourceDefaultProvider
        ]

        self._internal_default_label_sources = [
            ObjectDetectionLabelSourceDefaultProvider,
            ChipClassificationLabelSourceDefaultProvider,
            SemanticSegmentationLabelSourceDefaultProvider
        ]

        self._internal_default_label_stores = [
            ObjectDetectionGeoJSONStoreDefaultProvider,
            ChipClassificationGeoJSONStoreDefaultProvider,
            SemanticSegmentationRasterStoreDefaultProvider
        ]

        self._internal_default_evaluators = [
            ObjectDetectionEvaluatorDefaultProvider,
            ChipClassificationEvaluatorDefaultProvider,
            SemanticSegmentationEvaluatorDefaultProvider
        ]

        self.command_config_builders = {
            rv.ANALYZE: rv.command.AnalyzeCommandConfigBuilder,
            rv.CHIP: rv.command.ChipCommandConfigBuilder,
            rv.TRAIN: rv.command.TrainCommandConfigBuilder,
            rv.PREDICT: rv.command.PredictCommandConfigBuilder,
            rv.EVAL: rv.command.EvalCommandConfigBuilder,
            rv.BUNDLE: rv.command.BundleCommandConfigBuilder
        }

        self.aux_command_classes = {rv.COGIFY: rv.command.aux.CogifyCommand}

        self.commands = [
            rv.ANALYZE, rv.CHIP, rv.TRAIN, rv.PREDICT, rv.EVAL, rv.BUNDLE
        ]

        self.experiment_runners = {
            rv.INPROCESS: rv.runner.InProcessExperimentRunner,
            rv.AWS_BATCH: rv.runner.AwsBatchExperimentRunner,
            rv.LOCAL: rv.runner.LocalExperimentRunner
        }

        self.filesystems = [
            rvfs.HttpFileSystem,
            rvfs.S3FileSystem,
            # This is the catch-all case, ensure it's on the bottom of the search stack.
            rvfs.LocalFileSystem
        ]

    def initialize_config(self,
                          profile=None,
                          rv_home=None,
                          config_overrides=None,
                          verbosity=Verbosity.NORMAL):
        self._rv_config = RVConfig(
            profile=profile,
            rv_home=rv_home,
            config_overrides=config_overrides,
            verbosity=verbosity)
        # Reset the plugins in case this is a re-initialization,
        self._plugin_registry = None

    def _get_rv_config(self):
        """Returns the application configuration"""
        if self._rv_config is None:
            self.initialize_config()
        return self._rv_config

    def _ensure_plugins_loaded(self):
        if not self._plugin_registry:
            self._load_plugins()

    def _get_plugin_registry(self):
        self._ensure_plugins_loaded()
        return self._plugin_registry

    def _load_plugins(self):
        rv_config = self._get_rv_config()
        plugin_config = rv_config.get_subconfig('PLUGINS')
        self._plugin_registry = PluginRegistry(plugin_config,
                                               rv_config.rv_home)

    def get_config_builder(self, group, key):
        internal_builder = self._internal_config_builders.get((group, key))
        if internal_builder:
            return internal_builder
        else:
            self._ensure_plugins_loaded()
            plugin_builder = self._plugin_registry.config_builders.get((group,
                                                                        key))
            if plugin_builder:
                return plugin_builder

        if group == rv.BACKEND:
            if key in [
                    rv.TF_OBJECT_DETECTION, rv.TF_DEEPLAB,
                    rv.KERAS_CLASSIFICATION
            ]:
                raise RegistryError(
                    'Backend type {} not available. Perhaps you forgot to switch to '
                    'the TF Docker image.'.format(key))
            if key in [
                    rv.PYTORCH_CHIP_CLASSIFICATION,
                    rv.PYTORCH_SEMANTIC_SEGMENTATION,
                    rv.PYTORCH_OBJECT_DETECTION
            ]:
                raise RegistryError(
                    'Backend type {} not available. Perhaps you forgot to switch to '
                    'the PyTorch Docker image.'.format(key))

        raise RegistryError('Unknown type {} for {} '.format(key, group))

    def get_file_system(self, uri: str, mode: str = 'r',
                        search_plugins=True) -> rvfs.FileSystem:
        # If we are currently loading plugins, don't search for
        # plugin filesystems.
        if search_plugins:
            self._ensure_plugins_loaded()
            filesystems = (
                self._plugin_registry.filesystems + self.filesystems)
        else:
            filesystems = self.filesystems

        for fs in filesystems:
            if fs.matches_uri(uri, mode):
                return fs
        if mode == 'w':
            raise RegistryError('No matching filesystem to handle '
                                'writing to uri {}'.format(uri))
        else:
            raise RegistryError('No matching filesystem to handle '
                                'reading from uri {}'.format(uri))

    def get_raster_source_default_provider(self, s):
        """
        Gets the RasterSourceDefaultProvider for a given input string.
        """
        self._ensure_plugins_loaded()
        providers = (self._plugin_registry.default_raster_sources +
                     self._internal_default_raster_sources)

        for provider in providers:
            if provider.handles(s):
                return provider

        raise RegistryError(
            'No RasterSourceDefaultProvider found for {}'.format(s))

    def get_vector_source_default_provider(self, s):
        """
        Gets the VectorSourceDefaultProvider for a given input string.
        """
        self._ensure_plugins_loaded()
        providers = (self._plugin_registry.default_vector_sources +
                     self._internal_default_vector_sources)

        for provider in providers:
            if provider.handles(s):
                return provider

        raise RegistryError(
            'No VectorSourceDefaultProvider found for {}'.format(s))

    def get_label_source_default_provider(self, task_type, s):
        """
        Gets the RasterSourceDefaultProvider for a given input string.
        """
        self._ensure_plugins_loaded()
        providers = (self._plugin_registry.default_label_sources +
                     self._internal_default_label_sources)

        for provider in providers:
            if provider.handles(task_type, s):
                return provider

        raise RegistryError('No LabelSourceDefaultProvider '
                            'found for {} and task type {}'.format(
                                s, task_type))

    def get_label_store_default_provider(self, task_type, s=None):
        """
        Gets the RasterSourceDefaultProvider for a given input string.
        """

        self._ensure_plugins_loaded()
        providers = (self._plugin_registry.default_label_stores +
                     self._internal_default_label_stores)

        for provider in providers:
            if s:
                if provider.handles(task_type, s):
                    return provider
            else:
                if provider.is_default_for(task_type):
                    return provider

        if s:
            raise RegistryError('No LabelStoreDefaultProvider '
                                'found for {} and task type {}'.format(
                                    s, task_type))
        else:
            raise RegistryError('No LabelStoreDefaultProvider '
                                'found for task type {}'.format(task_type))

    def get_evaluator_default_provider(self, task_type):
        """
        Gets the EvaluatorDefaultProvider for a given task
        """

        self._ensure_plugins_loaded()
        providers = (self._plugin_registry.default_evaluators +
                     self._internal_default_evaluators)

        for provider in providers:
            if provider.is_default_for(task_type):
                return provider

        raise RegistryError('No EvaluatorDefaultProvider '
                            'found for task type {}'.format(task_type))

    def _get_aux_command_class(self, command_type):
        aux_command_class = self.aux_command_classes.get(command_type)

        if aux_command_class is None:
            self._ensure_plugins_loaded()
            aux_command_class = self._plugin_registry.aux_command_classes.get(
                command_type)

        return aux_command_class

    def get_command_config_builder(self, command_type):
        builder = self.command_config_builders.get(command_type)
        if builder is None:
            self._ensure_plugins_loaded()
            builder = self._plugin_registry.command_config_builders.get(
                command_type)

        if builder:

            def create_builder(*args, **kwargs):
                return builder(command_type, *args, **kwargs)

            return create_builder

        # Aux commands

        aux_command_class = self._get_aux_command_class(command_type)
        if aux_command_class:
            from rastervision.command.aux_command_config import AuxCommandConfigBuilder

            def create_builder(*args, **kwargs):
                return AuxCommandConfigBuilder(command_type, *args, **kwargs) \
                    .with_command_class(aux_command_class)

            return create_builder

        raise RegistryError(
            'No command found for type {}'.format(command_type))

    def get_aux_command_class(self, command_type):
        aux_command_class = self._get_aux_command_class(command_type)

        if aux_command_class:
            return aux_command_class
        else:
            raise RegistryError(
                'No command class found for type {}'.format(command_type))

    def get_commands(self):
        self._ensure_plugins_loaded()

        aux_commands_to_include = [
            command_type
            for command_type, cls in self.aux_command_classes.items()
            if cls.options.include_by_default
        ] + [
            command_type for command_type, cls in
            self._plugin_registry.aux_command_classes.items()
            if cls.options.include_by_default
        ]

        return self.commands + self._plugin_registry.commands + \
            aux_commands_to_include

    def get_experiment_runner(self, runner_type):
        internal_runner = self.experiment_runners.get(runner_type)
        if internal_runner:
            return internal_runner()
        else:
            self._ensure_plugins_loaded()
            plugin_runner = self._plugin_registry.experiment_runners.get(
                runner_type)
            if plugin_runner:
                return plugin_runner()
        raise RegistryError(
            'No experiment runner for type {}'.format(runner_type))

    def get_experiment_runner_keys(self):
        self._ensure_plugins_loaded()
        return (list(self.experiment_runners.keys()) + list(
            self._plugin_registry.experiment_runners.keys()))



def boxes_to_geojson(boxes, class_ids, crs_transformer, class_map,
                     scores=None):
    """Convert boxes and associated data into a GeoJSON dict.

    Args:
        boxes: list of Box in pixel row/col format.
        class_ids: list of int (one for each box)
        crs_transformer: CRSTransformer used to convert pixel coords to map
            coords in the GeoJSON
        class_map: ClassMap used to infer class_name from class_id
        scores: optional list of score or scores.
                If floats (one for each box), property name will be "score".
                If lists of floats, property name will be "scores".

    Returns:
        dict in GeoJSON format
    """
    features = []
    for box_ind, box in enumerate(boxes):
        polygon = box.geojson_coordinates()
        polygon = [list(crs_transformer.pixel_to_map(p)) for p in polygon]

        class_id = int(class_ids[box_ind])
        class_name = class_map.get_by_id(class_id).name

        feature = {
            'type': 'Feature',
            'geometry': {
                'type': 'Polygon',
                'coordinates': [polygon]
            },
            'properties': {
                'class_id': class_id,
                'class_name': class_name
            }
        }

        if scores is not None:
            box_scores = scores[box_ind]

            if box_scores is not None:
                if type(box_scores) is list:
                    feature['properties']['scores'] = box_scores
                else:
                    feature['properties']['score'] = box_scores

        features.append(feature)

    return {'type': 'FeatureCollection', 'features': features}



from abc import abstractmethod

import rastervision as rv
from rastervision.core import (Config, ConfigBuilder)


class AugmentorConfig(Config):
    def __init__(self, augmentor_type):
        self.augmentor_type = augmentor_type

    @abstractmethod
    def create_augmentor(self):
        """Create the Augmentor that this configuration represents"""
        pass

    def to_builder(self, augmentor_type):
        return rv._registry.get_config_builder(rv.AUGMENTOR,
                                               self.augmentor_type)(self)

    @staticmethod
    def builder(augmentor_type):
        return rv._registry.get_config_builder(rv.AUGMENTOR, augmentor_type)()

    @staticmethod
    def from_proto(msg):
        """Creates a AugmentorConfig from the specificed protobuf message
        """
        return rv._registry.get_config_builder(rv.AUGMENTOR, msg.augmentor_type)() \
                           .from_proto(msg) \
                           .build()


class AugmentorConfigBuilder(ConfigBuilder):
    pass



from abc import abstractmethod

import rastervision as rv
from rastervision.core.config import (Config, ConfigBuilder)
from rastervision.protos.label_store_pb2 import LabelStoreConfig as LabelStoreConfigMsg


class LabelStoreConfig(Config):
    def __init__(self, store_type):
        self.store_type = store_type

    def to_proto(self):
        msg = LabelStoreConfigMsg()
        msg.store_type = self.store_type
        return msg

    @abstractmethod
    def for_prediction(self, label_uri):
        """Build a copy of this config with a label_uri set.

        This is used in the Predictor to save labels to a specific path.
        """
        pass

    @abstractmethod
    def create_store(self, task_config, extent, crs_transformer, tmp_dir):
        """Create the Label Store for this configuration.

           Args:
              task_config: The TaskConfig for which this label source is supplying labels.
              crs_transformer: The crs_transformer used by the raster store this
                               label store is describing.
              tmp_dir: The temporary directory to use if files will need to be downloaded,
                       or None if only using local files.
        """
        pass

    def to_builder(self):
        return rv._registry.get_config_builder(rv.LABEL_STORE,
                                               self.store_type)(self)

    @staticmethod
    def builder(store_type):
        return rv._registry.get_config_builder(rv.LABEL_STORE, store_type)()

    @staticmethod
    def from_proto(msg):
        """Creates a TaskConfig from the specificed protobuf message
        """
        return rv._registry.get_config_builder(rv.LABEL_STORE, msg.store_type)() \
                           .from_proto(msg) \
                           .build()


class LabelStoreConfigBuilder(ConfigBuilder):
    pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/vector_source.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/vector_source.proto',
  package='rv.protos',
  syntax='proto2',
  serialized_pb=_b('\n\'rastervision/protos/vector_source.proto\x12\trv.protos\x1a\x1cgoogle/protobuf/struct.proto\"\xe5\x03\n\x12VectorSourceConfig\x12;\n\x07mbtiles\x18\x01 \x01(\x0b\x32(.rv.protos.VectorSourceConfig.VectorTileH\x00\x12\x38\n\x07geojson\x18\x02 \x01(\x0b\x32%.rv.protos.VectorSourceConfig.GeoJSONH\x00\x12\x30\n\rcustom_config\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x12\x13\n\x0bsource_type\x18\x04 \x02(\t\x12\x33\n\x12\x63lass_id_to_filter\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x18\n\x10\x64\x65\x66\x61ult_class_id\x18\x06 \x01(\x05\x12*\n\tline_bufs\x18\x07 \x01(\x0b\x32\x17.google.protobuf.Struct\x12+\n\npoint_bufs\x18\x08 \x01(\x0b\x32\x17.google.protobuf.Struct\x1a\x39\n\nVectorTile\x12\x0b\n\x03uri\x18\x01 \x02(\t\x12\x0c\n\x04zoom\x18\x02 \x02(\x05\x12\x10\n\x08id_field\x18\x03 \x02(\t\x1a\x16\n\x07GeoJSON\x12\x0b\n\x03uri\x18\x01 \x02(\tB\x16\n\x14vector_source_config')
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_VECTORSOURCECONFIG_VECTORTILE = _descriptor.Descriptor(
  name='VectorTile',
  full_name='rv.protos.VectorSourceConfig.VectorTile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='rv.protos.VectorSourceConfig.VectorTile.uri', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='zoom', full_name='rv.protos.VectorSourceConfig.VectorTile.zoom', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id_field', full_name='rv.protos.VectorSourceConfig.VectorTile.id_field', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=522,
)

_VECTORSOURCECONFIG_GEOJSON = _descriptor.Descriptor(
  name='GeoJSON',
  full_name='rv.protos.VectorSourceConfig.GeoJSON',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='rv.protos.VectorSourceConfig.GeoJSON.uri', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=524,
  serialized_end=546,
)

_VECTORSOURCECONFIG = _descriptor.Descriptor(
  name='VectorSourceConfig',
  full_name='rv.protos.VectorSourceConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mbtiles', full_name='rv.protos.VectorSourceConfig.mbtiles', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='geojson', full_name='rv.protos.VectorSourceConfig.geojson', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='custom_config', full_name='rv.protos.VectorSourceConfig.custom_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='source_type', full_name='rv.protos.VectorSourceConfig.source_type', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='class_id_to_filter', full_name='rv.protos.VectorSourceConfig.class_id_to_filter', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='default_class_id', full_name='rv.protos.VectorSourceConfig.default_class_id', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='line_bufs', full_name='rv.protos.VectorSourceConfig.line_bufs', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='point_bufs', full_name='rv.protos.VectorSourceConfig.point_bufs', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_VECTORSOURCECONFIG_VECTORTILE, _VECTORSOURCECONFIG_GEOJSON, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='vector_source_config', full_name='rv.protos.VectorSourceConfig.vector_source_config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=85,
  serialized_end=570,
)

_VECTORSOURCECONFIG_VECTORTILE.containing_type = _VECTORSOURCECONFIG
_VECTORSOURCECONFIG_GEOJSON.containing_type = _VECTORSOURCECONFIG
_VECTORSOURCECONFIG.fields_by_name['mbtiles'].message_type = _VECTORSOURCECONFIG_VECTORTILE
_VECTORSOURCECONFIG.fields_by_name['geojson'].message_type = _VECTORSOURCECONFIG_GEOJSON
_VECTORSOURCECONFIG.fields_by_name['custom_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_VECTORSOURCECONFIG.fields_by_name['class_id_to_filter'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_VECTORSOURCECONFIG.fields_by_name['line_bufs'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_VECTORSOURCECONFIG.fields_by_name['point_bufs'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_VECTORSOURCECONFIG.oneofs_by_name['vector_source_config'].fields.append(
  _VECTORSOURCECONFIG.fields_by_name['mbtiles'])
_VECTORSOURCECONFIG.fields_by_name['mbtiles'].containing_oneof = _VECTORSOURCECONFIG.oneofs_by_name['vector_source_config']
_VECTORSOURCECONFIG.oneofs_by_name['vector_source_config'].fields.append(
  _VECTORSOURCECONFIG.fields_by_name['geojson'])
_VECTORSOURCECONFIG.fields_by_name['geojson'].containing_oneof = _VECTORSOURCECONFIG.oneofs_by_name['vector_source_config']
_VECTORSOURCECONFIG.oneofs_by_name['vector_source_config'].fields.append(
  _VECTORSOURCECONFIG.fields_by_name['custom_config'])
_VECTORSOURCECONFIG.fields_by_name['custom_config'].containing_oneof = _VECTORSOURCECONFIG.oneofs_by_name['vector_source_config']
DESCRIPTOR.message_types_by_name['VectorSourceConfig'] = _VECTORSOURCECONFIG

VectorSourceConfig = _reflection.GeneratedProtocolMessageType('VectorSourceConfig', (_message.Message,), dict(

  VectorTile = _reflection.GeneratedProtocolMessageType('VectorTile', (_message.Message,), dict(
    DESCRIPTOR = _VECTORSOURCECONFIG_VECTORTILE,
    __module__ = 'rastervision.protos.vector_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.VectorSourceConfig.VectorTile)
    ))
  ,

  GeoJSON = _reflection.GeneratedProtocolMessageType('GeoJSON', (_message.Message,), dict(
    DESCRIPTOR = _VECTORSOURCECONFIG_GEOJSON,
    __module__ = 'rastervision.protos.vector_source_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.VectorSourceConfig.GeoJSON)
    ))
  ,
  DESCRIPTOR = _VECTORSOURCECONFIG,
  __module__ = 'rastervision.protos.vector_source_pb2'
  # @@protoc_insertion_point(class_scope:rv.protos.VectorSourceConfig)
  ))
_sym_db.RegisterMessage(VectorSourceConfig)
_sym_db.RegisterMessage(VectorSourceConfig.VectorTile)
_sym_db.RegisterMessage(VectorSourceConfig.GeoJSON)


# @@protoc_insertion_point(module_scope)



from abc import (ABC, abstractmethod)
import os

import rastervision as rv


class LabelStoreDefaultProvider(ABC):
    @staticmethod
    @abstractmethod
    def is_default_for(task_type):
        """Returns True if this label store is the default for this tasks_type"""
        pass

    @staticmethod
    @abstractmethod
    def handles(task_type, s):
        """Returns True of this provider is a default for this task_type and string"""
        pass

    @abstractmethod
    def construct(s=None):
        """Construts a default LabelStore based on the string.
        """
        pass


class ObjectDetectionGeoJSONStoreDefaultProvider(LabelStoreDefaultProvider):
    @staticmethod
    def is_default_for(task_type):
        return task_type == rv.OBJECT_DETECTION

    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.OBJECT_DETECTION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.json', '.geojson']
        return False

    @staticmethod
    def construct(uri=None):
        b = rv.LabelStoreConfig.builder(rv.OBJECT_DETECTION_GEOJSON)
        if uri:
            b = b.with_uri(uri)

        return b.build()


class ChipClassificationGeoJSONStoreDefaultProvider(LabelStoreDefaultProvider):
    @staticmethod
    def is_default_for(task_type):
        return task_type == rv.CHIP_CLASSIFICATION

    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.CHIP_CLASSIFICATION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.json', '.geojson']
        return False

    @staticmethod
    def construct(uri=None):
        b = rv.LabelStoreConfig.builder(rv.CHIP_CLASSIFICATION_GEOJSON)
        if uri:
            b = b.with_uri(uri)
        return b.build()


class SemanticSegmentationRasterStoreDefaultProvider(
        LabelStoreDefaultProvider):
    @staticmethod
    def is_default_for(task_type):
        return task_type == rv.SEMANTIC_SEGMENTATION

    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.SEMANTIC_SEGMENTATION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.tiff', '.tif']
        return False

    @staticmethod
    def construct(uri=None):
        b = rv.LabelStoreConfig.builder(rv.SEMANTIC_SEGMENTATION_RASTER)
        if uri:
            b = b.with_uri(uri)
        return b.build()



import os
import click

from rastervision.runner import ExperimentRunner
from rastervision.utils.files import save_json_config
from rastervision.cli import Verbosity


def make_command(command_config_uri, tmp_dir=None):
    verbosity = Verbosity.get()
    v_flag = 'v' * max(0, verbosity - 1)
    if v_flag:
        v_flag = '-{}'.format(v_flag)
    if tmp_dir is None:
        command = 'python -m rastervision {} run_command {}'.format(
            v_flag, command_config_uri)
    else:
        command = 'python -m rastervision {} run_command {} --tempdir {}'.format(
            v_flag, command_config_uri, tmp_dir)
    return command


class OutOfProcessExperimentRunner(ExperimentRunner):
    """A class implementing functionality for out-of-process running of experiments.

    The term "out-of-process" refers to the fact that experiments are
    not run within this process, but instead separate processes are
    forked for each stage.

    In the case of `AwsBatchExperimentRunner`, which derives from this
    class, those processes are run remotely on AWS.  In case of
    `LocalExperimentRunner`, which also derives from this class, the
    processes run locally.  This behavior can be contrasted with
    `InProcessExperimentRunner` wherein experiment stages are run
    within the calling process.

    """

    def __init__(self):
        self.tmp_dir = None

    def _run_experiment(self, command_dag):
        """Runs all commands."""

        ids_to_job = {}
        for command_id in command_dag.get_sorted_command_ids():
            command_def = command_dag.get_command_definition(command_id)
            command_config = command_def.command_config
            command_root_uri = command_config.root_uri
            command_basename = 'command-config-{}.json'.format(
                command_config.split_id)
            command_uri = os.path.join(command_root_uri, command_basename)
            print('Saving command configuration to {}...'.format(command_uri))
            save_json_config(command_config.to_proto(), command_uri)

            parent_job_ids = []
            for upstream_id in command_dag.get_upstream_command_ids(
                    command_id):
                if upstream_id not in ids_to_job:
                    cur_command = (command_config.command_type, command_id)
                    u = command_dag.get_command(upstream_id)
                    upstream_command = (u.command_type, upstream_id)
                    raise Exception(
                        '{} command has parent command of {}, '
                        'but does not exist in previous submissions - '
                        'topological sort on command_dag error.'.format(
                            cur_command, upstream_command))
                parent_job_ids.append(ids_to_job[upstream_id])

            run_command = make_command(command_uri, self.tmp_dir)
            job_id = self.submit(
                command_config.command_type,
                command_config.split_id,
                command_def.experiment_id,
                run_command,
                parent_job_ids,
                utilizes_gpu=command_config.utilizes_gpu())

            ids_to_job[command_id] = job_id

    def _dry_run(self, command_dag):
        """Runs all commands."""
        click.echo(
            click.style(
                '\n{} commands to be issued:'.format(
                    self.execution_environment),
                fg='green',
                bold=True,
                underline=True))
        for command_id in command_dag.get_sorted_command_ids():
            command_def = command_dag.get_command_definition(command_id)
            command_config = command_def.command_config
            command_root_uri = command_config.root_uri
            command_basename = 'command-config-{}.json'.format(
                command_config.split_id)
            command_uri = os.path.join(command_root_uri, command_basename)
            run_command = make_command(command_uri, self.tmp_dir)
            click.echo('  {}'.format(run_command))



from abc import (ABC, abstractmethod)
import os

import rastervision as rv


class LabelSourceDefaultProvider(ABC):
    @staticmethod
    @abstractmethod
    def handles(task_type, s):
        """Returns True of this provider is a default for this task_type and string"""
        pass

    @abstractmethod
    def construct(s):
        """Construts a default LabelSource based on the string.
        """
        pass


class ObjectDetectionLabelSourceDefaultProvider(LabelSourceDefaultProvider):
    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.OBJECT_DETECTION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.geojson', '.json', '.mbtiles']

    @staticmethod
    def construct(uri):
        return rv.LabelSourceConfig.builder(rv.OBJECT_DETECTION) \
                                   .with_uri(uri) \
                                   .build()


class ChipClassificationLabelSourceDefaultProvider(LabelSourceDefaultProvider):
    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.CHIP_CLASSIFICATION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.geojson', '.json', '.mbtiles']

    @staticmethod
    def construct(uri):
        return rv.LabelSourceConfig.builder(rv.CHIP_CLASSIFICATION) \
                                   .with_uri(uri) \
                                   .build()


class SemanticSegmentationLabelSourceDefaultProvider(
        LabelSourceDefaultProvider):
    @staticmethod
    def handles(task_type, uri):
        if task_type == rv.SEMANTIC_SEGMENTATION:
            ext = os.path.splitext(uri)[1]
            return ext.lower() in ['.tif', '.tiff']
        return False

    @staticmethod
    def construct(uri):
        return rv.LabelSourceConfig.builder(rv.SEMANTIC_SEGMENTATION) \
                                   .with_raster_source(uri) \
                                   .build()



from typing import Tuple

import numpy as np
from PIL import ImageColor


def color_to_triple(color: str) -> Tuple[int, int, int]:
    """Given a PIL ImageColor string, return a triple of integers
    representing the red, green, and blue values.

    Args:
         color: A PIL ImageColor string

    Returns:
         An triple of integers

    """
    if color is None:
        r = np.random.randint(0, 0x100)
        g = np.random.randint(0, 0x100)
        b = np.random.randint(0, 0x100)
        return (r, g, b)
    else:
        return ImageColor.getrgb(color)


def color_to_integer(color: str) -> int:
    """Given a PIL ImageColor string, return a packed integer.

    Args:
         color: A PIL ImageColor string

    Returns:
         An integer containing the packed RGB values.

    """
    triple = color_to_triple(color)
    r = triple[0] * (1 << 16)
    g = triple[1] * (1 << 8)
    b = triple[2] * (1 << 0)
    integer = r + g + b
    return integer


def rgb_to_int_array(rgb_array):
    r = np.array(rgb_array[:, :, 0], dtype=np.uint32) * (1 << 16)
    g = np.array(rgb_array[:, :, 1], dtype=np.uint32) * (1 << 8)
    b = np.array(rgb_array[:, :, 2], dtype=np.uint32) * (1 << 0)
    return r + g + b
